import functools
from spaceone.api.monitoring.v1 import data_source_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.monitoring.model.data_source_model import DataSource

__all__ = ["DataSourceInfo", "DataSourcesInfo"]


def PluginInfo(vo):
    if vo:
        info = {
            "plugin_id": vo.plugin_id,
            "version": vo.version,
            "options": change_struct_type(vo.options),
            "metadata": change_struct_type(vo.metadata),
            "secret_id": vo.secret_id,
            "provider": vo.provider,
            "upgrade_mode": vo.upgrade_mode,
        }

        return data_source_pb2.DataSourcePluginInfo(**info)
    else:
        return None


def DataSourceInfo(data_source_vo: DataSource, minimal=False):
    info = {
        "data_source_id": data_source_vo.data_source_id,
        "name": data_source_vo.name,
        "state": data_source_vo.state,
        "monitoring_type": data_source_vo.monitoring_type,
        "provider": data_source_vo.provider,
    }

    if not minimal:
        info.update(
            {
                "capability": change_struct_type(data_source_vo.capability),
                "plugin_info": PluginInfo(data_source_vo.plugin_info),
                "tags": change_struct_type(data_source_vo.tags),
                "domain_id": data_source_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(data_source_vo.created_at),
            }
        )

    return data_source_pb2.DataSourceInfo(**info)


def DataSourcesInfo(data_source_vos, total_count, **kwargs):
    return data_source_pb2.DataSourcesInfo(
        results=list(map(functools.partial(DataSourceInfo, **kwargs), data_source_vos)),
        total_count=total_count,
    )



import logging

from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.webhook_model import Webhook
from spaceone.monitoring.manager.plugin_manager import PluginManager
from spaceone.monitoring.connector.webhook_plugin_connector import (
    WebhookPluginConnector,
)

_LOGGER = logging.getLogger(__name__)


class WebhookPluginManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.wp_connector: WebhookPluginConnector = self.locator.get_connector(
            "WebhookPluginConnector"
        )

    def initialize(self, endpoint: str) -> None:
        _LOGGER.debug(f"[initialize] webhook plugin endpoint: {endpoint}")
        self.wp_connector.initialize(endpoint)

    def init_plugin(self, options):
        plugin_info = self.wp_connector.init(options)

        _LOGGER.debug(f"[plugin_info] {plugin_info}")
        plugin_metadata = plugin_info.get("metadata", {})

        return plugin_metadata

    def verify_plugin(self, options):
        self.wp_connector.verify(options)

    def parse_event(self, options, data):
        return self.wp_connector.parse_event(options, data)

    def get_webhook_plugin_endpoint_by_vo(self, webhook_vo: Webhook):
        plugin_info = webhook_vo.plugin_info.to_dict()
        endpoint, updated_version = self.get_webhook_plugin_endpoint(
            plugin_info, webhook_vo.domain_id
        )

        if updated_version:
            _LOGGER.debug(
                f'[get_webhook_plugin_endpoint_by_vo] upgrade plugin version: {plugin_info["version"]} -> {updated_version}'
            )
            self.upgrade_webhook_plugin_version(webhook_vo, endpoint, updated_version)

        return endpoint

    def get_webhook_plugin_endpoint(self, plugin_info, domain_id):
        plugin_mgr: PluginManager = self.locator.get_manager("PluginManager")
        return plugin_mgr.get_plugin_endpoint(plugin_info, domain_id)

    def upgrade_webhook_plugin_version(
        self, webhook_vo: Webhook, endpoint, updated_version
    ):
        plugin_info = webhook_vo.plugin_info.to_dict()
        self.initialize(endpoint)
        plugin_metadata = self.init_plugin(plugin_info.get("options", {}))
        plugin_info["version"] = updated_version
        plugin_info["metadata"] = plugin_metadata
        webhook_vo.update({"plugin_info": plugin_info})



from spaceone.api.monitoring.v1 import metric_pb2, metric_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Metric(BaseAPI, metric_pb2_grpc.MetricServicer):
    pb2 = metric_pb2
    pb2_grpc = metric_pb2_grpc

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("MetricService", metadata) as metric_service:
            return self.locator.get_info("MetricsInfo", metric_service.list(params))

    def get_data(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("MetricService", metadata) as metric_service:
            return self.locator.get_info(
                "MetricDataInfo", metric_service.get_data(params)
            )



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.core.plugin.server import PluginServer
from spaceone.monitoring.plugin.webhook.interface.grpc import app
from spaceone.monitoring.plugin.webhook.service.webhook_service import WebhookService
from spaceone.monitoring.plugin.webhook.service.event_service import EventService

__all__ = ["WebhookPluginServer"]


class WebhookPluginServer(PluginServer):
    _grpc_app: GRPCServer = app
    _global_conf_path: str = (
        "spaceone.monitoring.plugin.webhook.conf.global_conf:global_conf"
    )
    _plugin_methods = {
        "Webhook": {"service": WebhookService, "methods": ["init", "verify"]},
        "Event": {"service": EventService, "methods": ["parse"]},
    }



import logging

from spaceone.core import config
from spaceone.core.token import get_token
from spaceone.core.locator import Locator
from spaceone.core.scheduler import IntervalScheduler

_LOGGER = logging.getLogger(__name__)


class MonitoringAlertScheduler(IntervalScheduler):
    def __init__(self, queue, interval):
        super().__init__(queue, interval)
        self.locator = Locator()
        self._init_config()
        self._create_metadata()

    def _init_config(self):
        self._token = get_token("TOKEN")

    def _create_metadata(self):
        self._metadata = {
            "token": self._token,
            "service": "monitoring",
            "resource": "Job",
            "verb": "create_jobs_by_domain",
        }

    def create_task(self):
        stp = {
            "name": "monitoring_alert_schedule",
            "version": "v1",
            "executionEngine": "BaseWorker",
            "stages": [
                {
                    "locator": "SERVICE",
                    "name": "JobService",
                    "metadata": self._metadata,
                    "method": "create_jobs_by_domain",
                    "params": {"params": {}},
                }
            ],
        }

        return [stp]



import jsonschema
from schematics import Model
from schematics.exceptions import ValidationError
from schematics.types import BaseType, ListType, DictType, StringType
from schematics.types.compound import ModelType

__all__ = ["MetricPluginMetadataModel", "LogPluginMetadataModel"]


class JSONSchemaType(BaseType):
    def validate_netloc(self, value):
        try:
            jsonschema.Draft7Validator.check_schema(value)
        except Exception as e:
            raise ValidationError(
                key=f"Plugin metadata is invalid. (filter_format = {str(value)}"
            )


class DynamicField(Model):
    name = StringType(required=True)
    key = StringType(required=True)
    type = StringType()
    option = DictType(StringType)


class TemplateModel(Model):
    table = ListType(ModelType(DynamicField))


class MetricPluginMetadataModel(Model):
    supported_stat = ListType(StringType, required=True)
    required_keys = ListType(StringType)
    supported_providers = ListType(StringType)


class LogPluginMetadataModel(Model):
    required_keys = ListType(StringType)
    view = DictType(BaseType)
    supported_providers = ListType(StringType)



import logging
from spaceone.core.pygrpc import BaseAPI
from spaceone.api.monitoring.plugin import event_pb2, event_pb2_grpc
from spaceone.monitoring.plugin.webhook.service.event_service import EventService

_Logger = logging.getLogger("spaceone")


class Event(BaseAPI, event_pb2_grpc.EventServicer):
    pb2 = event_pb2
    pb2_grpc = event_pb2_grpc

    def parse(self, request, context):
        params, metadata = self.parse_request(request, context)
        event_svc = EventService(metadata)
        response = event_svc.parse(params)
        return self.dict_to_message(response)



import functools

from spaceone.api.monitoring.v1 import alert_pb2
from spaceone.core import utils
from spaceone.core.pygrpc.message_type import *

from spaceone.monitoring.model.alert_model import Alert

__all__ = ["AlertInfo", "AlertsInfo"]


def AlertResourceInfo(vo):
    if vo:
        info = {
            "resource_id": vo.resource_id,
            "resource_type": vo.resource_type,
            "name": vo.name,
        }

        return alert_pb2.AlertResource(**info)
    else:
        return None


def AlertInfo(alert_vo: Alert, minimal=False):
    info = {
        "alert_number": alert_vo.alert_number,
        "alert_id": alert_vo.alert_id,
        "title": alert_vo.title,
        "state": alert_vo.state,
        "assignee": alert_vo.assignee,
        "urgency": alert_vo.urgency,
        "escalation_step": alert_vo.escalation_step,
        "escalation_ttl": alert_vo.escalation_ttl,
        "project_id": alert_vo.project_id,
        "workspace_id": alert_vo.workspace_id,
    }

    if not minimal:
        info.update(
            {
                "description": alert_vo.description,
                "severity": alert_vo.severity,
                "rule": alert_vo.rule,
                "image_url": alert_vo.image_url,
                "resource": AlertResourceInfo(alert_vo.resource),
                "provider": alert_vo.provider,
                "account": alert_vo.account,
                "additional_info": change_struct_type(alert_vo.additional_info),
                "triggered_by": alert_vo.triggered_by,
                "webhook_id": alert_vo.webhook_id,
                "escalation_policy_id": alert_vo.escalation_policy_id,
                "domain_id": alert_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(alert_vo.created_at),
                "updated_at": utils.datetime_to_iso8601(alert_vo.updated_at),
                "acknowledged_at": utils.datetime_to_iso8601(alert_vo.acknowledged_at),
                "resolved_at": utils.datetime_to_iso8601(alert_vo.resolved_at),
                "escalated_at": utils.datetime_to_iso8601(alert_vo.escalated_at),
            }
        )

    return alert_pb2.AlertInfo(**info)


def AlertsInfo(alert_vos, total_count, **kwargs):
    return alert_pb2.AlertsInfo(
        results=list(map(functools.partial(AlertInfo, **kwargs), alert_vos)),
        total_count=total_count,
    )



import logging
from typing import Union
from spaceone.core.service import (
    BaseService,
    transaction,
    convert_model,
    change_timestamp_value,
)
from spaceone.monitoring.plugin.webhook.model.event.request import EventParseRequest
from spaceone.monitoring.plugin.webhook.model.event.response import EventsResponse

_LOGGER = logging.getLogger(__name__)


class EventService(BaseService):
    @transaction
    @convert_model
    def parse(self, params: EventParseRequest) -> Union[EventsResponse, dict]:
        """Parsing Event Webhook

        Args:
            params (EventRequest): {
                'options': 'dict',
                'data': 'dict'
            }

        Returns:
            Union[EventResponse, dict]
        """
        func = self.get_plugin_method("parse")
        return func(params.dict())



from spaceone.monitoring.error.alert import *
from spaceone.monitoring.error.data_source import *
from spaceone.monitoring.error.escalation_policy import *
from spaceone.monitoring.error.event_rule import *
from spaceone.monitoring.error.metric import *
from spaceone.monitoring.error.project_alert_config import *
from spaceone.monitoring.error.webhook import *



from spaceone.api.monitoring.v1 import alert_pb2, alert_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Alert(BaseAPI, alert_pb2_grpc.AlertServicer):
    pb2 = alert_pb2
    pb2_grpc = alert_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            return self.locator.get_info("AlertInfo", alert_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            return self.locator.get_info("AlertInfo", alert_service.update(params))

    def update_state(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            return self.locator.get_info(
                "AlertInfo", alert_service.update_state(params)
            )

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            alert_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            return self.locator.get_info("AlertInfo", alert_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            alert_vos, total_count = alert_service.list(params)
            return self.locator.get_info(
                "AlertsInfo", alert_vos, total_count, minimal=self.get_minimal(params)
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("AlertService", metadata) as alert_service:
            return self.locator.get_info("StatisticsInfo", alert_service.stat(params))



from spaceone.api.monitoring.v1 import (
    project_alert_config_pb2,
    project_alert_config_pb2_grpc,
)
from spaceone.core.pygrpc import BaseAPI


class ProjectAlertConfig(
    BaseAPI, project_alert_config_pb2_grpc.ProjectAlertConfigServicer
):
    pb2 = project_alert_config_pb2
    pb2_grpc = project_alert_config_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            return self.locator.get_info(
                "ProjectAlertConfigInfo", project_alert_config_service.create(params)
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            return self.locator.get_info(
                "ProjectAlertConfigInfo", project_alert_config_service.update(params)
            )

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            project_alert_config_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            return self.locator.get_info(
                "ProjectAlertConfigInfo", project_alert_config_service.get(params)
            )

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            project_alert_config_vos, total_count = project_alert_config_service.list(
                params
            )
            return self.locator.get_info(
                "ProjectAlertConfigsInfo",
                project_alert_config_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "ProjectAlertConfigService", metadata
        ) as project_alert_config_service:
            return self.locator.get_info(
                "StatisticsInfo", project_alert_config_service.stat(params)
            )



from typing import List
from spaceone.monitoring.plugin.webhook.lib.server import WebhookPluginServer

app = WebhookPluginServer()


@app.route("Webhook.init")
def webhook_init(params: dict) -> dict:
    """init plugin by options

    Args:
        params (WebhookInitRequest): {
            'options': 'dict'      # Required
        }

    Returns:
        WebhookResponse: {
            'metadata': 'dict'
        }
    """
    pass


@app.route("Webhook.verify")
def webhook_verify(params: dict) -> None:
    """Verifying webhook plugin

    Args:
        params (WebhookVerityRequest): {
            'options': 'dict',      # Required
            'secret_data': 'dict',  # Required
            'schema': 'str',
            'domain_id': 'str'      # Required
        }

    Returns:
        None
    """
    pass


@app.route("Event.parse")
def event_parse(params: dict) -> List[dict]:
    """Parsing Event Webhook

    Args:
        params (EventRequest): {
            'options': 'dict',  # Required
            'data': 'dict'      # Required
        }

    Returns:
        List[EventResponse]
        {
            'event_key': 'str',         # Required
            'event_type': 'str',        # Required
            'title': 'str',
            'description': 'str',
            'severity': 'str',
            'resource': 'dict',
            'rule': 'str',              # Required
            'occurred_at': 'datetime',  # Required
            'additional_info': 'dict',
            'image_url': 'str'
        }
    """
    pass



from spaceone.api.monitoring.v1 import log_pb2, log_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Log(BaseAPI, log_pb2_grpc.LogServicer):
    pb2 = log_pb2
    pb2_grpc = log_pb2_grpc

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("LogService", metadata) as log_service:
            return self.locator.get_info("LogDataInfo", log_service.list(params))



from spaceone.api.monitoring.v1 import event_rule_pb2, event_rule_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class EventRule(BaseAPI, event_rule_pb2_grpc.EventRuleServicer):
    pb2 = event_rule_pb2
    pb2_grpc = event_rule_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            return self.locator.get_info(
                "EventRuleInfo", event_rule_service.create(params)
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            return self.locator.get_info(
                "EventRuleInfo", event_rule_service.update(params)
            )

    def change_order(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            return self.locator.get_info(
                "EventRuleInfo", event_rule_service.change_order(params)
            )

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            event_rule_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            return self.locator.get_info(
                "EventRuleInfo", event_rule_service.get(params)
            )

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            event_rule_vos, total_count = event_rule_service.list(params)
            return self.locator.get_info(
                "EventRulesInfo",
                event_rule_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EventRuleService", metadata
        ) as event_rule_service:
            return self.locator.get_info(
                "StatisticsInfo", event_rule_service.stat(params)
            )



import logging

from spaceone.core.manager import BaseManager

from spaceone.monitoring.model.alert_model import Alert, AlertNumber

_LOGGER = logging.getLogger(__name__)


class AlertManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.alert_model: Alert = self.locator.get_model("Alert")
        self.alert_number_model: AlertNumber = self.locator.get_model("AlertNumber")

    def create_alert(self, params: dict) -> Alert:
        def _rollback(vo: Alert) -> None:
            _LOGGER.info(
                f"[create_alert._rollback] "
                f"Delete alert : #{str(vo.alert_number)} "
                f"({vo.alert_id})"
            )
            vo.delete()

        params["alert_number"] = self._get_alert_number(
            params["domain_id"], params["workspace_id"]
        )
        params["alert_number_str"] = str(params["alert_number"])
        alert_vo: Alert = self.alert_model.create(params)
        self.transaction.add_rollback(_rollback, alert_vo)

        return alert_vo

    def update_alert(self, params):
        alert_vo: Alert = self.get_alert(params["alert_id"], params["domain_id"])
        return self.update_alert_by_vo(params, alert_vo)

    def update_alert_by_vo(self, params: dict, alert_vo: Alert) -> Alert:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_alert_by_vo._rollback] Revert Data : "
                f'{old_data["alert_id"]}'
            )
            alert_vo.update(old_data)

        self.transaction.add_rollback(_rollback, alert_vo.to_dict())
        return alert_vo.update(params)

    def delete_alert(self, alert_id, domain_id, workspace_id, user_projects=None):
        alert_vo: Alert = self.get_alert(
            alert_id, domain_id, workspace_id, user_projects
        )
        alert_vo.delete()

    def get_alert(
        self,
        alert_id: str,
        domain_id: str,
        workspace_id: str = None,
        user_projects: list = None,
    ) -> Alert:
        conditions = {
            "alert_id": alert_id,
            "domain_id": domain_id,
        }

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if user_projects:
            conditions["project_id"] = user_projects
        return self.alert_model.get(**conditions)

    def list_alerts(self, query: dict) -> dict:
        return self.alert_model.query(**query)

    def stat_alerts(self, query: dict) -> dict:
        return self.alert_model.stat(**query)

    def _get_alert_number(self, domain_id: str, workspace_id: str) -> int:
        def _rollback(vo: AlertNumber):
            _LOGGER.info(f"[_get_alert_number._rollback] Decrement Number: {vo.next}")
            vo.decrement("next", 1)

        alert_number_vos = self.alert_number_model.filter(
            domain_id=domain_id, workspace_id=workspace_id
        )
        if alert_number_vos.count() > 0:
            account_number_vo = alert_number_vos[0].increment("next", 1)
            self.transaction.add_rollback(_rollback, account_number_vo)
        else:
            account_number_vo = self.alert_number_model.create(
                {"domain_id": domain_id, "workspace_id": workspace_id}
            )
            self.transaction.add_rollback(_rollback, account_number_vo)

        return account_number_vo.next



from spaceone.core.error import *


class ERROR_SAME_RESPONDER_ALREADY_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = "The same responder already exists. (resource_type = {resource_type}, resource_id = {resource_id})"


class ERROR_RESPONDER_NOT_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Responder does not exist. (resource_type = {resource_type}, resource_id = {resource_id})"


class ERROR_SAME_PROJECT_DEPENDENCY_ALREADY_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = "The same project already exists. (project_id = {project_id})"


class ERROR_PROJECT_DEPENDENCY_NOT_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Project does not exist. (project_id = {project_id})"


class ERROR_ALERT_ALREADY_PROCESSED(ERROR_INVALID_ARGUMENT):
    _message = "Alert has already been processed. (alert_id = {alert_id})"


class ERROR_MERGE_ALERT_NOT_EXIST(ERROR_INVALID_ARGUMENT):
    _message = "Alert does not exist in merge list. (alert_id = {alert_id})"



import logging

from spaceone.core.service import *

from spaceone.monitoring.error.escalation_policy import *
from spaceone.monitoring.manager.escalation_policy_manager import (
    EscalationPolicyManager,
)
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class EscalationPolicyService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.escalation_policy_mgr: EscalationPolicyManager = self.locator.get_manager(
            "EscalationPolicyManager"
        )

    @transaction(
        permission="monitoring:EscalationPolicy.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["name", "rules", "domain_id", "workspace_id"])
    def create(self, params: dict) -> EscalationPolicy:
        """Create escalation policy

        Args:
            params (dict): {
                'name': 'str',              # required
                'rules': 'list',            # required
                'repeat_count': 'int',
                'finish_condition': 'str',
                'project_id': 'str',
                'tags': 'dict',
                'resource_group': 'str',    # required
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth (required)
            }

        Returns:
            escalation_policy_vo (object)
        """
        resource_group = params["resource_group"]
        project_id = params.get("project_id")
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")

        if resource_group == "PROJECT":
            project_info = identity_mgr.get_project(project_id)
            params["workspace_id"] = project_info.get("workspace_id")
        else:
            identity_mgr.check_workspace(workspace_id, domain_id)
            params["project_id"] = "*"

        return self.escalation_policy_mgr.create_escalation_policy(params)

    @transaction(
        permission="monitoring:EscalationPolicy.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["escalation_policy_id", "domain_id", "workspace_id"])
    def update(self, params):
        """Update escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'dict',     # required
                'name': 'str',
                'rules': 'list',
                'repeat_count': 'int',
                'finish_condition': 'str',
                'tags': 'dict',
                'domain_id': 'str',                 # injected from auth (required)
                'workspace_id': 'str',              # injected from auth (required)
                'user_projects': 'list'             # injected from auth
            }

        Returns:
            escalation_policy_vo (object)
        """

        escalation_policy_id = params["escalation_policy_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        if "rules" in params:
            params["repeat_count"] = params.get("repeat_count", 0)

        escalation_policy_vo = self.escalation_policy_mgr.get_escalation_policy(
            escalation_policy_id, workspace_id, domain_id, user_projects
        )
        return self.escalation_policy_mgr.update_escalation_policy_by_vo(
            params, escalation_policy_vo
        )

    @transaction(
        permission="monitoring:EscalationPolicy.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["escalation_policy_id", "domain_id", "workspace_id"])
    def set_default(self, params):
        """Get escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',  # required
                'domain_id': 'str',             # injected from auth (required)
                'workspace_id': 'str',          # injected from auth (required)
                'user_projects': 'list'         # injected from auth
            }

        Returns:
            escalation_policy_vo (object)
        """

        escalation_policy_id = params["escalation_policy_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")

        escalation_policy_vo: EscalationPolicy = (
            self.escalation_policy_mgr.get_escalation_policy(
                escalation_policy_id, workspace_id, domain_id, user_projects
            )
        )

        if escalation_policy_vo.resource_group != "WORKSPACE":
            raise ERROR_INVALID_ESCALATION_POLICY_SCOPE(
                escalation_policy_id=escalation_policy_id
            )

        if escalation_policy_vo.is_default:
            return escalation_policy_vo
        else:
            return self.escalation_policy_mgr.set_default_escalation_policy(
                params, escalation_policy_vo
            )

    @transaction(
        permission="monitoring:EscalationPolicy.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["escalation_policy_id", "domain_id", "workspace_id"])
    def delete(self, params):
        """Delete escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',  # required
                'domain_id': 'str',             # injected from auth (required)
                'workspace_id': 'str',          # injected from auth (required)
                'user_projects': 'list'         # injected from auth
            }

        Returns:
            None
        """

        self.escalation_policy_mgr.delete_escalation_policy(
            params["escalation_policy_id"],
            params["workspace_id"],
            params["domain_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:EscalationPolicy.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["escalation_policy_id", "domain_id", "workspace_id"])
    def get(self, params):
        """Get escalation policy

        Args:
            params (dict): {
                'escalation_policy_id': 'str',  # required
                'domain_id': 'str',             # injected from auth (required)
                'workspace_id': 'str',          # injected from auth
                'user_projects': 'list'         # injected from auth
            }

        Returns:
            escalation_policy_vo (object)
        """

        return self.escalation_policy_mgr.get_escalation_policy(
            params["escalation_policy_id"],
            params["workspace_id"],
            params["domain_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:EscalationPolicy.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "escalation_policy_id",
            "name",
            "is_default",
            "finish_condition",
            "resource_group",
            "project_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["escalation_policy_id", "name"])
    def list(self, params):
        """List escalation polices

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'escalation_policy_id': 'str',
                'name': 'str',
                'is_default': 'bool',
                'finish_condition': 'str',
                'resource_group': 'str',
                'project_id': 'str',
                'domain_id': 'str',                            # injected from auth (required)
                'workspace_id': 'str',                         # injected from auth
                'user_projects': 'list'                        # injected from auth
            }

        Returns:
            escalation_policy_vos (object)
            total_count
        """
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        project_id = params.get("project_id")
        if not self.escalation_policy_mgr.is_default_escalation_policy(
            domain_id, workspace_id, project_id
        ):
            self.escalation_policy_mgr.create_default_escalation_policy(
                domain_id, workspace_id, project_id
            )

        query = params.get("query", {})
        return self.escalation_policy_mgr.list_escalation_policies(query)

    @transaction(
        permission="monitoring:EscalationPolicy.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @change_value_by_rule("APPEND", "workspace_id", "*")
    @change_value_by_rule("APPEND", "project_id", "*")
    @change_value_by_rule("APPEND", "user_projects", "*")
    @check_required(["query", "workspace_id", "domain_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["escalation_policy_id", "name"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',   # required
                'domain_id': 'str',                                       # injected from auth (required)
                'workspace_id': 'str',                                    # injected from auth
                'user_projects': 'list'                                   # injected from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.escalation_policy_mgr.stat_escalation_policies(query)



import logging
from datetime import datetime

from spaceone.core import cache
from spaceone.core.service import *

from spaceone.monitoring.error.alert import *
from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.event_manager import EventManager
from spaceone.monitoring.manager.job_manager import JobManager
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.model.alert_model import Alert
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class AlertService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.alert_mgr: AlertManager = self.locator.get_manager("AlertManager")
        self.event_mgr: EventManager = self.locator.get_manager("EventManager")

    @transaction(
        permission="monitoring:Alert.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["title", "project_id", "domain_id", "workspace_id"])
    def create(self, params: dict) -> Alert:
        """Create alert

        Args:
            params (dict): {
                'title': 'str',           # required
                'description': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'project_id': 'str',      # required
                'domain_id': 'str',       # injected from auth (required)
                'workspace_id': 'str',    # injected from auth (required)
            }

        Returns:
            alert_vo (object)
        """

        project_id = params["project_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]

        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager(
            "ProjectAlertConfigManager"
        )

        project_alert_config_vo: ProjectAlertConfig = (
            project_alert_config_mgr.get_project_alert_config(
                project_id, domain_id, workspace_id
            )
        )
        escalation_policy_vo: EscalationPolicy = (
            project_alert_config_vo.escalation_policy
        )

        params["escalation_policy_id"] = escalation_policy_vo.escalation_policy_id
        params["escalation_ttl"] = escalation_policy_vo.repeat_count + 1
        params["escalated_at"] = None

        # TODO: Check Assignee

        params["triggered_by"] = self.transaction.get_meta("authorization.user_id")

        alert_vo = self.alert_mgr.create_alert(params)

        self._create_notification(alert_vo, "create_alert_notification")

        return alert_vo

    @transaction(
        permission="monitoring:Alert.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["alert_id", "domain_id", "workspace_id"])
    def update(self, params):
        """Update alert

        Args:
            params (dict): {
                'alert_id': 'str',             # required
                'title': 'str',
                'state': 'str',
                'description': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'project_id': 'str',           # required
                'reset_description': 'bool',
                'reset_assignee': 'bool',
                'domain_id': 'str',            # injected from auth (required)
                'workspace_id': 'str',         # injected from auth (required)
                'user_projects': 'list',       # injected from auth
            }

        Returns:
            alert_vo (object)
        """

        alert_id = params["alert_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")
        project_id = params.get("project_id")
        state = params.get("state")
        assignee = params.get("assignee")
        reset_description = params.get("reset_description", False)
        reset_assignee = params.get("reset_assignee", False)

        is_resolved_notify = False
        is_assignee_notify = False

        if project_id:
            project_alert_config_mgr: ProjectAlertConfigManager = (
                self.locator.get_manager("ProjectAlertConfigManager")
            )

            project_alert_config_vo: ProjectAlertConfig = (
                project_alert_config_mgr.get_project_alert_config(
                    project_id, domain_id, workspace_id
                )
            )
            escalation_policy_vo: EscalationPolicy = (
                project_alert_config_vo.escalation_policy
            )

            params["escalation_policy_id"] = escalation_policy_vo.escalation_policy_id
            params["escalation_ttl"] = escalation_policy_vo.repeat_count
            params["escalation_step"] = 1
            params["escalated_at"] = None
            params["assignee"] = None
            assignee = None

        if state:
            if state == "ACKNOWLEDGED":
                params["acknowledged_at"] = datetime.utcnow()
                params["resolved_at"] = None
            elif state == "RESOLVED":
                params["escalation_ttl"] = 0
                params["resolved_at"] = datetime.utcnow()
            elif state == "TRIGGERED":
                params["acknowledged_at"] = None
                params["resolved_at"] = None

        alert_vo = self.alert_mgr.get_alert(
            alert_id, domain_id, workspace_id, user_projects
        )

        if alert_vo.state == "ERROR":
            raise ERROR_INVALID_PARAMETER(
                key="state", reason="The error state cannot be changed."
            )

        if reset_description:
            params["description"] = ""

        if reset_assignee:
            params["assignee"] = None
            assignee = None

        if assignee:
            # TODO: Check Assignee
            is_assignee_notify = True

        if alert_vo.state in ["TRIGGERED", "ACKNOWLEDGED"] and state == "RESOLVED":
            is_resolved_notify = True

        updated_alert_vo: Alert = self.alert_mgr.update_alert_by_vo(params, alert_vo)

        if is_resolved_notify:
            self._create_notification(updated_alert_vo, "create_resolved_notification")
        elif is_assignee_notify and updated_alert_vo.state in [
            "TRIGGERED",
            "ACKNOWLEDGED",
        ]:
            self._create_notification(
                updated_alert_vo, "create_assigned_notification", assignee
            )

        return updated_alert_vo

    @transaction(
        permission="monitoring:Alert.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["alert_id", "domain_id", "workspace_id"])
    def assign_user(self, params):
        """Assign user to alert

        Args:
            params (dict): {
                'alert_id': 'str',             # required
                'assignee': 'str',             # required
                'domain_id': 'str',            # injected from auth (required)
                'workspace_id': 'str',         # injected from auth (required)
                'user_projects': 'list',       # injected from auth
            }

        Returns:
            alert_vo (object)
        """

        alert_id = params["alert_id"]
        domain_id = params["domain_id"]
        workspace_id = params["workspace_id"]
        user_projects = params.get("user_projects")
        assignee = params["assignee"]

        alert_vo = self.alert_mgr.get_alert(
            alert_id, domain_id, workspace_id, user_projects
        )
        # TODO: Not Implemented
        pass

    @transaction(exclude=["authentication", "authorization", "mutation"])
    @check_required(["alert_id", "access_key", "state"])
    def update_state(self, params):
        """Update alert state

        Args:
            params (dict): {
                'alert_id': 'str',            # required
                'access_key': 'str',          # required
                'state': 'str'                # required
            }

        Returns:
            alert_vo (object)
        """
        alert_id = params["alert_id"]
        access_key = params["access_key"]
        state = params["state"]

        is_resolved_notify = False

        # Check Access Key
        domain_id = self._check_access_key(alert_id, access_key)

        # Check State
        self._check_state(state)

        alert_vo = self.alert_mgr.get_alert(alert_id, domain_id)

        if alert_vo.state == "ERROR":
            raise ERROR_INVALID_PARAMETER(
                key="state", reason="The error state cannot be changed."
            )

        if alert_vo.state != "TRIGGERED":
            # raise ERROR_ALERT_ALREADY_PROCESSED(alert_id=alert_id)
            return alert_vo

        update_params = {"state": state}

        if state == "ACKNOWLEDGED":
            update_params["acknowledged_at"] = datetime.utcnow()
        elif state == "RESOLVED":
            update_params["resolved_at"] = datetime.utcnow()

        if alert_vo.state in ["TRIGGERED", "ACKNOWLEDGED"] and state == "RESOLVED":
            is_resolved_notify = True

        updated_alert_vo = self.alert_mgr.update_alert_by_vo(update_params, alert_vo)

        if is_resolved_notify:
            self._create_notification(updated_alert_vo, "create_resolved_notification")

        return updated_alert_vo

    @transaction(
        permission="monitoring:Alert.write",
        role_types=["WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["alert_id", "domain_id", "workspace_id"])
    def delete(self, params):
        """Delete alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str',            # injected from auth (required)
                'workspace_id': 'str',         # injected from auth (required)
                'user_projects': 'list',       # injected from auth
            }

        Returns:
            None
        """

        self.alert_mgr.delete_alert(
            params["alert_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Alert.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["alert_id", "domain_id", "workspace_id"])
    def get(self, params):
        """Get alert

        Args:
            params (dict): {
                'alert_id': 'str',
                'domain_id': 'str',            # injected from auth (required)
                'workspace_id': 'str',         # injected from auth (required)
                'user_projects': 'list',       # injected from auth
            }

        Returns:
            alert_vo (object)
        """

        return self.alert_mgr.get_alert(
            params["alert_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )

    @transaction(
        permission="monitoring:Alert.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "alert_number",
            "alert_id",
            "title",
            "state",
            "assignee",
            "urgency",
            "severity",
            "resource_id",
            "triggered_by",
            "webhook_id",
            "escalation_policy_id",
            "project_id",
            "domain_id",
            "workspace_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["alert_id", "alert_number_str", "title"])
    def list(self, params):
        """List alerts

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'alert_number': 'str',
                'alert_id': 'str',
                'title': 'str',
                'state': 'str',
                'assignee': 'str',
                'urgency': 'str',
                'severity': 'str',
                'resource_id': 'str',
                'webhook_id': 'bool',
                'escalation_policy_id': 'str',
                'project_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',                             # injected from auth (required)
                'user_projects': 'list'                         # injected from auth
            }

        Returns:
            alert_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.alert_mgr.list_alerts(query)

    @transaction(
        permission="monitoring:Alert.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id", "workspace_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["alert_id", "title"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str',            # injected from auth (required)
                'workspace_id': 'str',         # injected from auth (required)
                'user_projects': 'list',       # injected from auth
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.alert_mgr.stat_alerts(query)

    def _create_notification(
        self, alert_vo: Alert, method: str, user_id: str = None
    ) -> None:
        params = {
            "alert_id": alert_vo.alert_id,
            "domain_id": alert_vo.domain_id,
            "workspace_id": alert_vo.workspace_id,
        }

        if user_id:
            params["user_id"] = user_id

        job_mgr: JobManager = self.locator.get_manager("JobManager")
        job_mgr.push_task(
            "monitoring_alert_notification_from_manual", "JobService", method, params
        )

    @staticmethod
    def _check_access_key(alert_id: str, access_key: str) -> str:
        domain_id = cache.get(f"alert-notification-callback:{alert_id}:{access_key}")

        if domain_id is None:
            raise ERROR_PERMISSION_DENIED()

        return domain_id

    @staticmethod
    def _check_state(state: str) -> None:
        if state not in ["ACKNOWLEDGED", "RESOLVED"]:
            raise ERROR_INVALID_PARAMETER(
                key="state", reason="Unsupported state. (ACKNOWLEDGED | RESOLVED)"
            )



from spaceone.api.monitoring.v1 import log_pb2
from spaceone.core.pygrpc.message_type import *

__all__ = ["LogDataInfo"]


def LogDataInfo(log_data):
    info = {
        "results": change_list_value_type(log_data["results"]),
        "domain_id": log_data["domain_id"],
    }

    return log_pb2.LogDataInfo(**info)



from spaceone.api.monitoring.v1 import webhook_pb2, webhook_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Webhook(BaseAPI, webhook_pb2_grpc.WebhookServicer):
    pb2 = webhook_pb2
    pb2_grpc = webhook_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("WebhookInfo", webhook_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("WebhookInfo", webhook_service.update(params))

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("WebhookInfo", webhook_service.enable(params))

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("WebhookInfo", webhook_service.disable(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            webhook_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def update_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info(
                "WebhookInfo", webhook_service.update_plugin(params)
            )

    def verify_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            webhook_service.verify_plugin(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("WebhookInfo", webhook_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            webhook_vos, total_count = webhook_service.list(params)
            return self.locator.get_info(
                "WebhooksInfo",
                webhook_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("WebhookService", metadata) as webhook_service:
            return self.locator.get_info("StatisticsInfo", webhook_service.stat(params))



DEFAULT_ESCALATION_POLICY = {
    "name": "Default",
    "is_default": True,
    "rules": [{"notification_level": "ALL", "escalate_minutes": 0}],
    "repeat_count": 0,
    "finish_condition": "ACKNOWLEDGED",
}



from spaceone.monitoring.info.alert_info import *
from spaceone.monitoring.info.common_info import *
from spaceone.monitoring.info.data_source_info import *
from spaceone.monitoring.info.escalation_policy_info import *
from spaceone.monitoring.info.event_info import *
from spaceone.monitoring.info.event_rule_info import *
from spaceone.monitoring.info.log_info import *
from spaceone.monitoring.info.metric_info import *
from spaceone.monitoring.info.note_info import *
from spaceone.monitoring.info.project_alert_config_info import *
from spaceone.monitoring.info.webhook_info import *



ROUTER = [
    {
        "router_path": "spaceone.monitoring.interface.rest.v1.alert:router",
        "router_options": {
            "prefix": "/monitoring/v1",
        },
    },
    {
        "router_path": "spaceone.monitoring.interface.rest.v1.event:router",
        "router_options": {
            "prefix": "/monitoring/v1",
        },
    },
]



import logging

from spaceone.core.manager import BaseManager

from spaceone.monitoring.model.note_model import Note

_LOGGER = logging.getLogger(__name__)


class NoteManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.note_model: Note = self.locator.get_model("Note")

    def create_note(self, params: dict) -> Note:
        def _rollback(vo):
            _LOGGER.info(f"[create_note._rollback] " f"Delete note : {vo.note_id}")
            vo.delete()

        note_vo: Note = self.note_model.create(params)
        self.transaction.add_rollback(_rollback, note_vo)

        return note_vo

    def update_note(self, params: dict) -> Note:
        note_vo: Note = self.get_note(
            params["note_id"],
            params["domain_id"],
            params["workspace_id"],
            params.get("user_projects"),
        )
        return self.update_note_by_vo(params, note_vo)

    def update_note_by_vo(self, params: dict, note_vo: Note) -> Note:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_note_by_vo._rollback] Revert Data : " f'{old_data["note_id"]}'
            )
            note_vo.update(old_data)

        self.transaction.add_rollback(_rollback, note_vo.to_dict())

        return note_vo.update(params)

    def delete_note(
        self,
        note_id: str,
        domain_id: str,
        workspace_id: str,
        user_projects: list = None,
    ):
        note_vo: Note = self.get_note(note_id, domain_id, workspace_id, user_projects)
        note_vo.delete()

    def get_note(
        self,
        note_id: str,
        domain_id: str,
        workspace_id: str,
        user_projects: list = None,
    ):
        conditions = {
            "note_id": note_id,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
        }

        if user_projects:
            conditions["project_id"] = user_projects

        return self.note_model.get(
            note_id=note_id,
            domain_id=domain_id,
            workspace_id=workspace_id,
            user_projects=user_projects,
        )

    def list_notes(self, query: dict) -> dict:
        return self.note_model.query(**query)

    def stat_notes(self, query: dict) -> dict:
        return self.note_model.stat(**query)



import logging
from typing import List
from datetime import datetime, timedelta

from spaceone.core.error import *
from spaceone.core import queue, utils, config
from spaceone.core.manager import BaseManager
from spaceone.monitoring.model.job_model import Job

_LOGGER = logging.getLogger(__name__)


class JobManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.job_model: Job = self.locator.get_model("Job")
        self.job_timeout = config.get_global("JOB_TIMEOUT", 600)

    def is_domain_job_running(self, domain_id):
        job_vos: List[Job] = self.job_model.filter(
            domain_id=domain_id, status="IN_PROGRESS"
        )

        running_job_count = job_vos.count()

        for job_vo in job_vos:
            if job_vo.remained_tasks == 0 and job_vo.status not in ["ERROR", "TIMEOUT"]:
                self.change_success_status(job_vo)
                running_job_count -= 1

            if datetime.utcnow() > (
                job_vo.created_at + timedelta(seconds=self.job_timeout)
            ):
                self.change_timeout_status(job_vo)
                running_job_count -= 1

        if running_job_count > 0:
            return True
        else:
            return False

    def create_job(self, domain_id):
        return self.job_model.create({"domain_id": domain_id})

    def get_job(self, job_id, domain_id):
        return self.job_model.get(job_id=job_id, domain_id=domain_id)

    @staticmethod
    def decrease_remained_tasks(job_vo: Job):
        job_vo.decrement("remained_tasks", 1)

    @staticmethod
    def change_success_status(job_vo: Job):
        job_vo.delete()
        # job_vo.update({
        #     'status': 'SUCCESS',
        #     'finished_at': datetime.utcnow()
        # })

    @staticmethod
    def change_timeout_status(job_vo: Job):
        _LOGGER.error(f"Job Timeout ({job_vo.job_id}): {job_vo.domain_id}")

        job_vo.update({"status": "TIMEOUT", "finished_at": datetime.utcnow()})

    @staticmethod
    def change_error_status(job_vo: Job, e):
        if not isinstance(e, ERROR_BASE):
            e = ERROR_UNKNOWN(message=str(e))

        _LOGGER.error(f"Job Error ({job_vo.job_id}): {e.message}", exc_info=True)

        job_vo.update({"status": "ERROR", "finished_at": datetime.utcnow()})

        job_vo.append("errors", {"error_code": e.error_code, "message": e.message})

    def push_task(self, task_name, class_name, method, params):
        task = {
            "name": task_name,
            "version": "v1",
            "executionEngine": "BaseWorker",
            "stages": [
                {
                    "locator": "SERVICE",
                    "name": class_name,
                    "metadata": self.transaction.meta,
                    "method": method,
                    "params": {"params": params},
                }
            ],
        }

        queue.put("monitoring_q", utils.dump_json(task))



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class EventRuleCondition(EmbeddedDocument):
    key = StringField(required=True)
    value = StringField(required=True)
    operator = StringField(choices=("eq", "contain", "not", "not_contain"))


class EventRuleOptions(EmbeddedDocument):
    stop_processing = BooleanField(default=False)


class EventRule(MongoModel):
    event_rule_id = StringField(max_length=40, generate_id="er", unique=True)
    name = StringField(max_length=255, default="")
    order = IntField(required=True)
    conditions = ListField(EmbeddedDocumentField(EventRuleCondition))
    conditions_policy = StringField(max_length=20, choices=("ALL", "ANY", "ALWAYS"))
    actions = DictField()
    options = EmbeddedDocumentField(EventRuleOptions, default=EventRuleOptions)
    tags = DictField()
    resource_group = StringField(max_length=40, choices=("WORKSPACE", "PROJECT"))
    project_id = StringField(max_length=40, default=None, null=True)
    domain_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "order",
            "conditions",
            "conditions_policy",
            "actions",
            "options",
            "tags",
        ],
        "minimal_fields": [
            "event_rule_id",
            "name",
            "order",
            "resource_group",
            "project_id",
            "workspace_id",
        ],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["order"],
        "indexes": [
            # 'event_rule_id',
            "order",
            "conditions_policy",
            "resource_group",
            "project_id",
            "workspace_id",
            "domain_id",
        ],
    }



import functools
from typing import List

from spaceone.api.monitoring.v1 import escalation_policy_pb2
from spaceone.core import utils
from spaceone.core.pygrpc.message_type import *

from spaceone.monitoring.model.escalation_policy_model import (
    EscalationPolicy,
    EscalationRule,
)

__all__ = ["EscalationPolicyInfo", "EscalationPoliciesInfo"]


def EscalationRulesInfo(vos: List[EscalationRule]):
    if vos:
        rules = []
        for vo in vos:
            info = {
                "notification_level": vo.notification_level,
                "escalate_minutes": vo.escalate_minutes,
            }
            rule = escalation_policy_pb2.EscalationPolicyRule(**info)
            rules.append(rule)

        return rules
    else:
        return None


def EscalationPolicyInfo(escalation_policy_vo: EscalationPolicy, minimal=False):
    info = {
        "escalation_policy_id": escalation_policy_vo.escalation_policy_id,
        "name": escalation_policy_vo.name,
        "is_default": escalation_policy_vo.is_default,
        "project_id": escalation_policy_vo.project_id,
    }

    if not minimal:
        info.update(
            {
                "rules": EscalationRulesInfo(escalation_policy_vo.rules),
                "repeat_count": escalation_policy_vo.repeat_count,
                "finish_condition": escalation_policy_vo.finish_condition,
                "tags": change_struct_type(escalation_policy_vo.tags),
                "resource_group": escalation_policy_vo.resource_group,
                "workspace_id": escalation_policy_vo.workspace_id,
                "domain_id": escalation_policy_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(
                    escalation_policy_vo.created_at
                ),
            }
        )

    return escalation_policy_pb2.EscalationPolicyInfo(**info)


def EscalationPoliciesInfo(escalation_policy_vos, total_count, **kwargs):
    return escalation_policy_pb2.EscalationPoliciesInfo(
        results=list(
            map(
                functools.partial(EscalationPolicyInfo, **kwargs), escalation_policy_vos
            )
        ),
        total_count=total_count,
    )



from spaceone.core.pygrpc.server import GRPCServer
from spaceone.monitoring.plugin.webhook.interface.grpc.event import Event
from spaceone.monitoring.plugin.webhook.interface.grpc.webhook import Webhook

_all_ = ["app"]

app = GRPCServer()
app.add_service(Event)
app.add_service(Webhook)



import logging

from google.protobuf.json_format import MessageToDict
from spaceone.core import pygrpc
from spaceone.core.connector import BaseConnector
from spaceone.core.utils import parse_endpoint

__all__ = ["DataSourcePluginConnector"]

_LOGGER = logging.getLogger(__name__)


class DataSourcePluginConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.client = None

    def initialize(self, endpoint):
        static_endpoint = self.config.get("endpoint")

        if static_endpoint:
            endpoint = static_endpoint

        e = parse_endpoint(endpoint)
        self.client = pygrpc.client(
            endpoint=f'{e.get("hostname")}:{e.get("port")}', version="plugin"
        )

    def init(self, options):
        response = self.client.DataSource.init(
            {
                "options": options,
            },
            metadata=self.transaction.get_connection_meta(),
        )

        return self._change_message(response)

    def verify(self, options, secret_data, schema=None):
        params = {"options": options, "secret_data": secret_data}

        if schema:
            params.update({"schema": schema})

        self.client.DataSource.verify(
            params, metadata=self.transaction.get_connection_meta()
        )

    def list_metrics(self, schema, options, secret_data, query):
        params = {"options": options, "secret_data": secret_data, "query": query}

        if schema:
            params.update({"schema": schema})

        response = self.client.Metric.list(
            params, metadata=self.transaction.get_connection_meta()
        )
        return self._change_message(response)

    def get_metric_data(self, params):
        response = self.client.Metric.get_data(
            params, metadata=self.transaction.get_connection_meta()
        )
        return self._change_message(response)

    def list_logs(
        self, schema, options, secret_data, query, keyword, start, end, sort, limit
    ):
        params = {
            "options": options,
            "secret_data": secret_data,
            "query": query,
            "start": start,
            "end": end,
        }

        if keyword:
            params.update({"keyword": keyword})

        if schema:
            params.update({"schema": schema})

        if sort:
            params.update({"sort": sort})

        if limit:
            params.update({"limit": limit})

        responses = self.client.Log.list(
            params, metadata=self.transaction.get_connection_meta()
        )

        for response in responses:
            yield self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=40)
    version = StringField(max_length=255)
    options = DictField(default={})
    metadata = DictField(default={})
    upgrade_mode = StringField(
        max_length=255, choices=("AUTO", "MANUAL"), default="AUTO"
    )

    def to_dict(self):
        return dict(self.to_mongo())


class Webhook(MongoModel):
    webhook_id = StringField(max_length=40, generate_id="webhook", unique=True)
    name = StringField(
        max_length=255, unique_with=["project_id", "workspace_id", "domain_id"]
    )
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED")
    )
    access_key = StringField(max_length=255)
    webhook_url = StringField(max_length=255)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default=None, null=True)
    tags = DictField()
    project = ReferenceField("ProjectAlertConfig", reverse_delete_rule=CASCADE)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "state",
            "access_key",
            "webhook_url",
            "capability",
            "plugin_info",
            "tags",
        ],
        "minimal_fields": [
            "webhook_id",
            "name",
            "state",
            "webhook_url",
            "project_id",
            "workspace_id",
        ],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["name"],
        "indexes": [
            # 'webhook_id',
            "state",
            "access_key",
            "project_id",
            "workspace_id",
            "domain_id",
        ],
    }



import logging

from spaceone.core.manager import BaseManager

from spaceone.monitoring.model.data_source_model import DataSource

_LOGGER = logging.getLogger(__name__)


class DataSourceManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.data_source_model: DataSource = self.locator.get_model("DataSource")

    def register_data_source(self, params: dict) -> DataSource:
        def _rollback(vo: DataSource):
            _LOGGER.info(
                f"[register_data_source._rollback] "
                f"Delete data source : {vo.name} "
                f"({vo.data_source_id})"
            )
            vo.delete()

        data_source_vo: DataSource = self.data_source_model.create(params)
        self.transaction.add_rollback(_rollback, data_source_vo)

        return data_source_vo

    def update_data_source(self, params):
        data_source_vo: DataSource = self.get_data_source(
            params["data_source_id"], params["domain_id"]
        )
        return self.update_data_source_by_vo(params, data_source_vo)

    def update_data_source_by_vo(
        self, params: dict, data_source_vo: DataSource
    ) -> DataSource:
        def _rollback(old_data: dict) -> None:
            _LOGGER.info(
                f"[update_data_source_by_vo._rollback] Revert Data : "
                f'{old_data["data_source_id"]}'
            )
            data_source_vo.update(old_data)

        self.transaction.add_rollback(_rollback, data_source_vo.to_dict())
        return data_source_vo.update(params)

    def deregister_data_source(self, data_source_id: str, domain_id: str) -> None:
        data_source_vo: DataSource = self.get_data_source(data_source_id, domain_id)
        data_source_vo.delete()

    def get_data_source(self, data_source_id: str, domain_id: str) -> DataSource:
        return self.data_source_model.get(
            data_source_id=data_source_id, domain_id=domain_id
        )

    def list_data_sources(self, query: dict) -> dict:
        return self.data_source_model.query(**query)

    def stat_data_sources(self, query: dict) -> dict:
        return self.data_source_model.stat(**query)



import copy
import logging

from spaceone.core import utils, cache, config
from spaceone.core.service import *

from spaceone.monitoring.error.webhook import *
from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.event_manager import EventManager
from spaceone.monitoring.manager.event_rule_manager import EventRuleManager
from spaceone.monitoring.manager.job_manager import JobManager
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.manager.webhook_manager import WebhookManager
from spaceone.monitoring.manager.webhook_plugin_manager import WebhookPluginManager
from spaceone.monitoring.model.alert_model import Alert
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.model.event_model import Event
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig
from spaceone.monitoring.model.webhook_model import Webhook

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class EventService(BaseService):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.resource = "Event"
        self.event_mgr: EventManager = self.locator.get_manager("EventManager")
        self.webhook_mgr: WebhookManager = self.locator.get_manager("WebhookManager")

    @transaction(exclude=["authentication", "authorization", "mutation"])
    @check_required(["webhook_id", "access_key", "data"])
    def create(self, params: dict) -> None:
        """Create event

        Args:
            params (dict): {
                'webhook_id': 'str',    # required
                'access_key': 'str',    # required
                'data': 'str'           # required
            }

        Returns:
            event_vo (object)
        """

        webhook_data = self._get_webhook_data(params["webhook_id"])

        self._check_access_key(params["access_key"], webhook_data["access_key"])
        self._check_webhook_state(webhook_data)

        try:
            webhook_plugin_mgr: WebhookPluginManager = self.locator.get_manager(
                "WebhookPluginManager"
            )
            endpoint, updated_version = webhook_plugin_mgr.get_webhook_plugin_endpoint(
                {
                    "plugin_id": webhook_data["plugin_id"],
                    "version": webhook_data["plugin_version"],
                    "upgrade_mode": webhook_data["plugin_upgrade_mode"],
                },
                webhook_data["domain_id"],
            )

            if updated_version:
                _LOGGER.debug(
                    f'[create] upgrade plugin version: {webhook_data["plugin_version"]} -> {updated_version}'
                )
                webhook_vo: Webhook = self.webhook_mgr.get_webhook(
                    webhook_data["webhook_id"], webhook_data["domain_id"]
                )
                webhook_plugin_mgr.upgrade_webhook_plugin_version(
                    webhook_vo, endpoint, updated_version
                )
                cache.delete(f'webhook-data:{webhook_data["webhook_id"]}')

            webhook_plugin_mgr.initialize(endpoint)
            response = webhook_plugin_mgr.parse_event(
                webhook_data["plugin_options"], params["data"]
            )

        except Exception as e:
            if not isinstance(e, ERROR_BASE):
                e = ERROR_UNKNOWN(message=str(e))

            _LOGGER.error(f"[create] Event parsing failed: {e.message}", exc_info=True)
            response = self._create_error_event(webhook_data["name"], e.message)

        for event_data in response.get("results", []):
            # TODO: Check event data using schematics

            _LOGGER.debug(f"[Event.create] event_data: {event_data}")
            self._create_event(event_data, params["data"], webhook_data)

    @transaction(
        permission="monitoring:Event.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["event_id", "domain_id"])
    def get(self, params):
        """Get event

        Args:
            params (dict): {
                'event_id': 'str',    # required
                'domain_id': 'str',   # injected from auth (required)
            }

        Returns:
            event_vo (object)
        """

        event_id = params["event_id"]
        domain_id = params["domain_id"]
        workspace_id = params.get("workspace_id")
        user_projects = params.get("user_projects")

        return self.event_mgr.get_event(
            event_id, domain_id, workspace_id, user_projects
        )

    @transaction(
        permission="monitoring:Event.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["domain_id"])
    @append_query_filter(
        [
            "event_id",
            "event_key",
            "event_type",
            "severity",
            "resource_id",
            "alert_id",
            "webhook_id",
            "project_id",
            "workspace_id" "domain_id",
            "user_projects",
        ]
    )
    @append_keyword_filter(["event_id", "title"])
    def list(self, params):
        """List events

        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.Query)',
                'event_id': 'str',
                'event_key': 'str',
                'event_type': 'str',
                'severity': 'str',
                'resource_id': 'str',
                'alert_id': 'str',
                'webhook_id': 'str',
                'project_id': 'str',
                'workspace_id': 'str',
                'domain_id': 'str',         # injected from auth (required)
                'user_projects': 'list',    # injected from auth
            }

        Returns:
            event_vos (object)
            total_count
        """

        query = params.get("query", {})
        return self.event_mgr.list_events(query)

    @transaction(
        permission="monitoring:Event.read",
        role_types=["DOMAIN_ADMIN", "WORKSPACE_OWNER", "WORKSPACE_MEMBER"],
    )
    @check_required(["query", "domain_id"])
    @append_query_filter(["domain_id", "workspace_id", "user_projects"])
    @append_keyword_filter(["event_id", "title"])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'domain_id': 'str',         # injected from auth (required)
                'workspace_id': 'str',      # injected from auth
                'user_projects': 'list',    # injected from auth

            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get("query", {})
        return self.event_mgr.stat_events(query)

    @cache.cacheable(key="webhook-data:{webhook_id}", expire=300)
    def _get_webhook_data(self, webhook_id):
        webhook_vo: Webhook = self.webhook_mgr.get_webhook_by_id(webhook_id)
        return {
            "webhook_id": webhook_vo.webhook_id,
            "name": webhook_vo.name,
            "project_id": webhook_vo.project_id,
            "domain_id": webhook_vo.domain_id,
            "state": webhook_vo.state,
            "access_key": webhook_vo.access_key,
            "plugin_id": webhook_vo.plugin_info.plugin_id,
            "plugin_version": webhook_vo.plugin_info.version,
            "plugin_upgrade_mode": webhook_vo.plugin_info.upgrade_mode,
            "plugin_options": webhook_vo.plugin_info.options,
        }

    @staticmethod
    def _check_access_key(request_access_key, webhook_access_key):
        if request_access_key != webhook_access_key:
            raise ERROR_PERMISSION_DENIED()

    @staticmethod
    def _check_webhook_state(webhook_data):
        if webhook_data["state"] == "DISABLED":
            raise ERROR_WEBHOOK_STATE_DISABLED(webhook_id=webhook_data["webhook_id"])

    def _create_event(self, event_data, raw_data, webhook_data):
        event_data["raw_data"] = copy.deepcopy(raw_data)
        event_data["occurred_at"] = utils.iso8601_to_datetime(
            event_data.get("occurred_at")
        )
        event_data["webhook_id"] = webhook_data["webhook_id"]
        event_data["project_id"] = webhook_data["project_id"]
        event_data["workspace_id"] = webhook_data["workspace_id"]
        event_data["domain_id"] = webhook_data["domain_id"]
        event_data["severity"] = event_data.get("severity", "NONE")
        event_data["account"] = event_data.get("account", "")
        event_data["provider"] = event_data.get("provider", "")

        event_rule_mgr: EventRuleManager = self.locator.get_manager("EventRuleManager")

        # Change event data by event rule
        event_data = event_rule_mgr.change_event_data(
            event_data,
            webhook_data["project_id"],
            webhook_data["domain_id"],
            webhook_data["workspace_id"],
        )

        event_vo: Event = self.event_mgr.get_event_by_key(
            event_data["event_key"],
            event_data["domain_id"],
            event_data["project_id"],
            event_data["workspace_id"],
        )

        if event_vo and event_vo.alert.state != "RESOLVED":
            self._update_alert(event_vo.alert, event_data)

            event_data["alert_id"] = event_vo.alert_id
            event_data["alert"] = event_vo.alert
        else:
            # Skip health event
            if event_data["event_type"] == "RECOVERY":
                _LOGGER.debug(
                    f'[_create_event] Skip health event: {event_data.get("title")} (event_type = RECOVERY)'
                )
                return None

            # Create new alert
            _LOGGER.debug(
                f'[_create_event] Create new alert: {event_data.get("title")} '
                f'(event_type = {event_data["event_type"]})'
            )
            alert_vo = self._create_alert(event_data)

            event_data["alert_id"] = alert_vo.alert_id
            event_data["alert"] = alert_vo

        self.event_mgr.create_event(event_data)

    def _create_alert(self, event_data):
        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")

        alert_data = copy.deepcopy(event_data)

        if "urgency" in event_data:
            alert_data["urgency"] = event_data["urgency"]
        else:
            alert_data["urgency"] = self._get_urgency_from_severity(
                event_data["severity"]
            )

        escalation_policy_id, escalation_ttl = self._get_escalation_policy_info(
            event_data["project_id"], event_data["domain_id"]
        )

        alert_data["escalation_policy_id"] = escalation_policy_id

        if event_data.get("no_notification", False):
            alert_data["escalation_ttl"] = 0
        else:
            alert_data["escalation_ttl"] = escalation_ttl + 1

        alert_data["triggered_by"] = alert_data["webhook_id"]

        if event_data.get("event_type", "ERROR") == "ERROR":
            alert_data["state"] = "ERROR"

        alert_vo = alert_mgr.create_alert(alert_data)

        self._create_notification(
            alert_vo.alert_id, alert_vo.domain_id, "create_alert_notification"
        )

        return alert_vo

    @staticmethod
    def _get_urgency_from_severity(severity):
        if severity in ["CRITICAL", "ERROR", "NOT_AVAILABLE"]:
            return "HIGH"
        else:
            return "LOW"

    @cache.cacheable(key="escalation-policy-info:{domain_id}:{project_id}", expire=300)
    def _get_escalation_policy_info(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(
            project_id, domain_id
        )
        escalation_policy_vo: EscalationPolicy = (
            project_alert_config_vo.escalation_policy
        )

        return (
            escalation_policy_vo.escalation_policy_id,
            escalation_policy_vo.repeat_count,
        )

    def _update_alert(self, alert_vo: Alert, event_data: dict) -> None:
        # Resolve alert when receiving recovery event
        alert_id = alert_vo.alert_id
        domain_id = alert_vo.domain_id

        alert_mgr: AlertManager = self.locator.get_manager("AlertManager")
        update_params = {"description": event_data.get("description", "")}

        is_resolved = self._check_resolved_state(event_data, alert_vo)

        if is_resolved:
            update_params["state"] = "RESOLVED"

        alert_mgr.update_alert_by_vo(update_params, alert_vo)

        if is_resolved:
            self._create_notification(
                alert_id, domain_id, "create_resolved_notification"
            )

    def _check_resolved_state(self, event_data: dict, alert_vo: Alert) -> bool:
        if (
            event_data["event_type"] == "RECOVERY"
            and alert_vo.state != "RESOLVED"
            and self._is_auto_recovery(alert_vo.project_id, alert_vo.domain_id)
        ):
            return True
        else:
            return False

    @cache.cacheable(key="auto-recovery:{domain_id}:{project_id}", expire=300)
    def _is_auto_recovery(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(
            project_id, domain_id
        )
        return project_alert_config_vo.options.recovery_mode == "AUTO"

    def _get_project_alert_config(self, project_id, domain_id):
        project_alert_config_mgr: ProjectAlertConfigManager = self.locator.get_manager(
            "ProjectAlertConfigManager"
        )
        return project_alert_config_mgr.get_project_alert_config(project_id, domain_id)

    def _get_notification_urgency(self, project_id, domain_id):
        project_alert_config_vo: ProjectAlertConfig = self._get_project_alert_config(
            project_id, domain_id
        )
        return project_alert_config_vo.options.notification_urgency

    def _create_notification(self, alert_id, domain_id, method):
        # if alert_vo.state != 'ERROR':
        self._set_transaction_token()

        job_mgr: JobManager = self.locator.get_manager("JobManager")
        job_mgr.push_task(
            "monitoring_alert_notification_from_webhook",
            "JobService",
            method,
            {"alert_id": alert_id, "domain_id": domain_id},
        )

    def _set_transaction_token(self):
        self.transaction.set_meta("token", config.get_global("TOKEN"))

    @staticmethod
    def _create_error_event(webhook_name, error_message):
        response = {
            "results": [
                {
                    "event_key": utils.generate_id("error"),
                    "event_type": "ERROR",
                    "title": f"Webhook Event Parsing Error - {webhook_name}",
                    "description": error_message,
                    "severity": "CRITICAL",
                }
            ]
        }

        return response



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class Note(MongoModel):
    note_id = StringField(max_length=40, generate_id="note", unique=True)
    note = StringField()
    alert_id = StringField(max_length=40)
    alert = ReferenceField("Alert", reverse_delete_rule=CASCADE)
    created_by = StringField(max_length=40)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": ["note", "project_id"],
        "minimal_fields": ["note_id", "note", "alert_id", "created_by", "workspace_id"],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["-created_at"],
        "indexes": [
            # 'note_id',
            "alert_id",
            "created_by",
            "workspace_id",
            "domain_id",
        ],
    }



import logging
from datetime import datetime, timedelta

from spaceone.core import config
from spaceone.core.manager import BaseManager

from spaceone.monitoring.model.event_model import Event

_LOGGER = logging.getLogger(__name__)


class EventManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event_model: Event = self.locator.get_model("Event")

    def create_event(self, params):
        def _rollback(event_vo):
            _LOGGER.info(
                f"[create_event._rollback] " f"Delete event : {event_vo.event_id}"
            )
            event_vo.delete()

        event_vo: Event = self.event_model.create(params)
        self.transaction.add_rollback(_rollback, event_vo)

        return event_vo

    def update_event(self, params):
        event_vo: Event = self.get_event(params["event_id"], params["domain_id"])
        return self.update_event_by_vo(params, event_vo)

    def update_event_by_vo(self, params, event_vo):
        def _rollback(old_data):
            _LOGGER.info(
                f"[update_event_by_vo._rollback] Revert Data : "
                f'{old_data["event_id"]}'
            )
            event_vo.update(old_data)

        self.transaction.add_rollback(_rollback, event_vo.to_dict())

        return event_vo.update(params)

    def delete_event(self, event_id, domain_id):
        event_vo: Event = self.get_event(event_id, domain_id)
        event_vo.delete()

    def get_event(self, event_id, domain_id, workspace_id=None, user_projects=None):
        conditions = {"event_id": event_id, "domain_id": domain_id}

        if workspace_id:
            conditions["workspace_id"] = workspace_id

        if user_projects:
            conditions["project_id"] = user_projects

        return self.event_model.get(**conditions)

    def list_events(self, query: dict) -> dict:
        return self.event_model.query(**query)

    def stat_events(self, query: dict) -> dict:
        return self.event_model.stat(**query)

    def get_event_by_key(self, event_key, domain_id, project_id, workspace_id):
        same_event_time = config.get_global("SAME_EVENT_TIME", 600)
        same_event_datetime = datetime.utcnow() - timedelta(seconds=same_event_time)

        query = {
            "filter": [
                {"k": "event_key", "v": event_key, "o": "eq"},
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "workspace_id", "v": workspace_id, "o": "eq"},
                {"k": "project_id", "v": project_id, "o": "eq"},
                {"k": "event_type", "v": "RECOVERY", "o": "not"},
                {"k": "created_at", "v": same_event_datetime, "o": "gte"},
            ],
            "sort": {"key": "created_at", "desc": True},
        }

        event_vos, total_count = self.list_events(query)
        if event_vos.count() > 0:
            return event_vos[0]
        else:
            return None



from spaceone.api.monitoring.v1 import event_pb2, event_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class Event(BaseAPI, event_pb2_grpc.EventServicer):
    pb2 = event_pb2
    pb2_grpc = event_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("EventService", metadata) as event_service:
            event_service.create(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("EventService", metadata) as event_service:
            return self.locator.get_info("EventInfo", event_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("EventService", metadata) as event_service:
            event_vos, total_count = event_service.list(params)
            return self.locator.get_info(
                "EventsInfo", event_vos, total_count, minimal=self.get_minimal(params)
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service("EventService", metadata) as event_service:
            return self.locator.get_info("StatisticsInfo", event_service.stat(params))



from spaceone.monitoring.service.alert_service import AlertService
from spaceone.monitoring.service.data_source_service import DataSourceService
from spaceone.monitoring.service.escalation_policy_service import (
    EscalationPolicyService,
)
from spaceone.monitoring.service.event_rule_service import EventRuleService
from spaceone.monitoring.service.event_service import EventService
from spaceone.monitoring.service.job_service import JobService
from spaceone.monitoring.service.log_service import LogService
from spaceone.monitoring.service.metric_service import MetricService
from spaceone.monitoring.service.note_service import NoteService
from spaceone.monitoring.service.project_alert_config_service import (
    ProjectAlertConfigService,
)
from spaceone.monitoring.service.webhook_service import WebhookService



import logging

from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.core.manager import BaseManager

_LOGGER = logging.getLogger(__name__)


class PluginManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plugin_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="plugin"
        )

    def get_plugin_endpoint(self, plugin_info: dict, domain_id: str) -> (str, str):
        plugin_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="plugin"
        )
        response = plugin_connector.dispatch(
            "Plugin.get_plugin_endpoint",
            {
                "plugin_id": plugin_info["plugin_id"],
                "version": plugin_info.get("version"),
                "upgrade_mode": plugin_info.get("upgrade_mode", "AUTO"),
                "domain_id": domain_id,
            },
        )

        _LOGGER.debug(f"[get_plugin_endpoint] response: {response}")
        return response["endpoint"], response.get("updated_version")



from spaceone.core.error import *


class ERROR_INVALID_PLUGIN_VERSION(ERROR_INVALID_ARGUMENT):
    _message = (
        "Plugin version is invalid. (plugin_id = {plugin_id}, version = {version})"
    )


class ERROR_SUPPORTED_SECRETS_NOT_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = "There are no secrets that support plugins. (plugin_id = {plugin_id}, provider = {provider})"


class ERROR_RESOURCE_SECRETS_NOT_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = "There are no secrets in the resources. (resource_id = {resource_id})"


class ERROR_WRONG_PLUGIN_SETTINGS(ERROR_BASE):
    _message = "The plugin settings is incorrect. (key = {key})"


class ERROR_INVALID_PLUGIN_OPTIONS(ERROR_INTERNAL_API):
    _message = "The options received from the plugin is invalid. (reason = {reason})"


class ERROR_DATA_SOURCE_STATE_DISABLED(ERROR_INVALID_ARGUMENT):
    _message = "Data source state is disabled. (data_source_id = {data_source_id})"


class ERROR_REQUIRED_KEYS_NOT_EXISTS(ERROR_INVALID_ARGUMENT):
    _message = (
        "There are no required keys in plugins metadata. (plugin_id = {plugin_id})"
    )



from datetime import datetime
from typing import Union

from mongoengine import *
from pydantic import BaseModel, Field
from spaceone.core.model.mongo_model import MongoModel


class EventResource(EmbeddedDocument):
    resource_id = StringField(default=None, null=True)
    resource_type = StringField(default=None, null=True)
    name = StringField(default=None, null=True)


class Event(MongoModel):
    event_id = StringField(max_length=40, generate_id="event", unique=True)
    event_key = StringField()
    event_type = StringField(
        max_length=20, default="ALERT", choices=("ALERT", "RECOVERY", "ERROR")
    )
    title = StringField()
    description = StringField(default=None, null=True)
    severity = StringField(
        max_length=20,
        default="NONE",
        choices=("CRITICAL", "ERROR", "WARNING", "INFO", "NOT_AVAILABLE", "NONE"),
    )
    rule = StringField(default=None, null=True)
    image_url = StringField(default=None, null=True)
    resource = EmbeddedDocumentField(EventResource, default=None, null=True)
    provider = StringField(default=None, null=True)
    account = StringField(default=None, null=True)
    raw_data = DictField()
    additional_info = DictField()
    alert = ReferenceField("Alert", reverse_delete_rule=CASCADE)
    alert_id = StringField(max_length=40)
    webhook_id = StringField(max_length=40)
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)
    occurred_at = DateTimeField(default=None, null=True)

    meta = {
        "updatable_fields": ["alert_id", "project_id", "alert"],
        "minimal_fields": [
            "event_id",
            "event_key",
            "event_type",
            "title",
            "severity",
            "alert_id",
            "project_id",
        ],
        "change_query_keys": {
            "user_projects": "project_id",
            "resource_id": "resource.resource_id",
        },
        "ordering": ["-created_at"],
        "indexes": [
            # 'event_id',
            "event_key",
            "event_type",
            "severity",
            "resource.resource_id",
            "resource.resource_type",
            "resource.name",
            "provider",
            "account",
            "alert",
            "alert_id",
            "webhook_id",
            "project_id",
            "workspace_id",
            "domain_id",
            "created_at",
            "occurred_at",
        ],
    }


# Pydantic Model


class CreateEventResource(BaseModel):
    resource_id: Union[str, None] = Field(None)
    resource_type: Union[str, None] = Field(None)
    name: Union[str, None] = Field(None)


class CreateEvent(BaseModel):
    event_key: Union[str, None] = Field(None)
    event_type: Union[str, None] = Field(None)
    title: Union[str, None] = Field(None)
    description: Union[str, None] = Field(None)
    severity: Union[str, None] = Field(None)
    rule: Union[str, None] = Field(None)
    image_url: Union[str, None] = Field(None)
    resource: Union[CreateEventResource, None] = Field(None)
    additional_info: Union[dict, None] = Field(None)
    project_id: Union[str, None] = Field(None)
    occurred_at: Union[datetime, None] = Field(None)



from spaceone.api.monitoring.v1 import data_source_pb2, data_source_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class DataSource(BaseAPI, data_source_pb2_grpc.DataSourceServicer):
    pb2 = data_source_pb2
    pb2_grpc = data_source_pb2_grpc

    def register(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.register(params)
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.update(params)
            )

    def enable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.enable(params)
            )

    def disable(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.disable(params)
            )

    def deregister(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            data_source_service.deregister(params)
            return self.locator.get_info("EmptyInfo")

    def update_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.update_plugin(params)
            )

    def verify_plugin(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            data_source_service.verify_plugin(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "DataSourceInfo", data_source_service.get(params)
            )

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            data_source_vos, total_count = data_source_service.list(params)
            return self.locator.get_info(
                "DataSourcesInfo",
                data_source_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "DataSourceService", metadata
        ) as data_source_service:
            return self.locator.get_info(
                "StatisticsInfo", data_source_service.stat(params)
            )



from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel

from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy


class AlertOptions(EmbeddedDocument):
    notification_urgency = StringField(
        max_length=20, default="ALL", choices=("ALL", "HIGH_ONLY")
    )
    recovery_mode = StringField(
        max_length=20, default="MANUAL", choices=("MANUAL", "AUTO")
    )

    def to_dict(self):
        return self.to_mongo()


class ProjectAlertConfig(MongoModel):
    project_id = StringField(max_length=40, unique=True)
    options = EmbeddedDocumentField(AlertOptions, default=AlertOptions)
    escalation_policy = ReferenceField("EscalationPolicy", reverse_delete_rule=DENY)
    escalation_policy_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": ["options", "escalation_policy", "escalation_policy_id"],
        "minimal_fields": ["project_id", "workspace_id", "options"],
        "change_query_keys": {"user_projects": "project_id"},
        "reference_query_keys": {"escalation_policy": EscalationPolicy},
        "indexes": [
            # 'project_id',
            "escalation_policy",
            "escalation_policy_id",
            "workspace_id",
            "domain_id",
        ],
    }



import logging

from spaceone.core import config
from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.core.manager import BaseManager

_LOGGER = logging.getLogger(__name__)

_GET_RESOURCE_METHODS = {
    "identity.Project": "get_project",
    "identity.ServiceAccount": "get_service_account",
}


class IdentityManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="identity", token=config.get_global("TOKEN")
        )

    def get_domain(self, domain_id):
        return self.identity_connector.dispatch("Domain.get", {"domain_id": domain_id})

    def get_user(self, user_id, domain_id):
        return self.identity_connector.dispatch(
            "User.get", {"user_id": user_id, "domain_id": domain_id}
        )

    def get_project(self, project_id: str, domain_id: str = None) -> dict:
        conditions = {"project_id": project_id}

        if domain_id:
            conditions["domain_id"] = domain_id

        return self.identity_connector.dispatch("Project.get", conditions)

    def check_workspace(self, workspace_id: str, domain_id: str) -> dict:
        return self.identity_connector.dispatch(
            "Workspace.check", {"workspace_id": workspace_id, "domain_id": domain_id}
        )

    def get_service_account(self, service_account_id, domain_id):
        return self.identity_connector.dispatch(
            "ServiceAccount.get",
            {"service_account_id": service_account_id, "domain_id": domain_id},
        )

    def get_resource(self, resource_type, resource_id, domain_id):
        if resource_type == "identity.Project":
            return self.get_project(resource_id)
        elif resource_type == "identity.ServiceAccount":
            return self.get_service_account(resource_id, domain_id)

    def list_service_accounts(self, query, domain_id):
        return self.identity_connector.dispatch(
            "ServiceAccount.list", {"query": query, "domain_id": domain_id}
        )



from spaceone.core.error import *


class ERROR_INVALID_ESCALATION_POLICY(ERROR_INVALID_ARGUMENT):
    _message = (
        "Escalation policy is invalid. (escalation_policy_id = {escalation_policy_id})"
    )


class ERROR_ALERT_FEATURE_IS_NOT_ACTIVATED(ERROR_INVALID_ARGUMENT):
    _message = "The alert feature is not activated in the project. (project_id = {project_id}, workspace_id = {workspace_id})"



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class EscalationRule(EmbeddedDocument):
    notification_level = StringField(max_length=20, default="ALL")
    escalate_minutes = IntField(default=0, min_value=0)

    def to_dict(self):
        return self.to_mongo()


class EscalationPolicy(MongoModel):
    escalation_policy_id = StringField(max_length=40, generate_id="ep", unique=True)
    name = StringField(max_length=255, unique_with="workspace_id")
    is_default = BooleanField(default=False)
    rules = ListField(EmbeddedDocumentField(EscalationRule))
    repeat_count = IntField(default=0, min_value=0)
    finish_condition = StringField(
        max_length=20, default="ACKNOWLEDGED", choices=("ACKNOWLEDGED", "RESOLVED")
    )
    tags = DictField()
    resource_group = StringField(max_length=40, choices=("WORKSPACE", "PROJECT"))
    project_id = StringField(max_length=40)
    workspace_id = StringField(max_length=40)
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": [
            "name",
            "is_default",
            "rules",
            "repeat_count",
            "finish_condition",
            "tags",
        ],
        "minimal_fields": ["escalation_policy_id", "name", "is_default", "project_id"],
        "change_query_keys": {"user_projects": "project_id"},
        "ordering": ["name"],
        "indexes": [
            # 'escalation_policy_id',
            "is_default",
            "finish_condition",
            "resource_group",
            "project_id",
            "workspace_id",
            "domain_id",
        ],
    }



from mongoengine import *

from spaceone.core.model.mongo_model import MongoModel


class PluginInfo(EmbeddedDocument):
    plugin_id = StringField(max_length=40)
    version = StringField(max_length=255)
    options = DictField(default={})
    metadata = DictField(default={})
    secret_id = StringField(max_length=40, default=None, null=True)
    provider = StringField(max_length=40, default=None, null=True)
    upgrade_mode = StringField(
        max_length=255, choices=("AUTO", "MANUAL"), default="AUTO"
    )

    def to_dict(self):
        return dict(self.to_mongo())


class DataSource(MongoModel):
    data_source_id = StringField(max_length=40, generate_id="ds", unique=True)
    name = StringField(max_length=255, unique_with="domain_id")
    state = StringField(
        max_length=20, default="ENABLED", choices=("ENABLED", "DISABLED")
    )
    monitoring_type = StringField(max_length=20, choices=("METRIC", "LOG"))
    provider = StringField(max_length=40, default=None, null=True)
    capability = DictField()
    plugin_info = EmbeddedDocumentField(PluginInfo, default=None, null=True)
    tags = DictField()
    domain_id = StringField(max_length=40)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        "updatable_fields": ["name", "state", "capability", "plugin_info", "tags"],
        "minimal_fields": [
            "data_source_id",
            "name",
            "state",
            "monitoring_type",
            "provider",
        ],
        "ordering": ["name"],
        "indexes": [
            # 'data_source_id',
            "state",
            "monitoring_type",
            "provider",
            "domain_id",
        ],
    }



import logging

from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.core.manager import BaseManager

from spaceone.monitoring.error import *

_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="secret"
        )

    def get_secret_from_resource(self, resource, data_source_vo, domain_id):
        secret = {}

        if data_source_plugin_info := data_source_vo.plugin_info:
            if secret_id := data_source_plugin_info.secret_id:
                secret = self.list_secrets_from_query(
                    {"secret_id": secret_id}, domain_id
                )[0]

        if not secret:
            resource_secrets = []
            for collection_info in resource.get("collection_info", []):
                if secret_id := collection_info.get("secret_id"):
                    resource_secrets.append(secret_id)

            if data_source_vo.capability.get("use_resource_secret", False):
                secret_filter = {
                    "provider": data_source_vo.plugin_info.provider,
                    "supported_schema": data_source_vo.capability.get(
                        "supported_schema", []
                    ),
                    "secrets": resource_secrets,
                }
                secret = self.list_secrets_from_query(secret_filter, domain_id)[0]

        return secret

    def list_secrets(self, query: dict, domain_id: str) -> dict:
        return self.secret_connector.dispatch(
            "Secret.list", {"query": query, "domain_id": domain_id}
        )

    def get_secret_data(self, secret_id, domain_id):
        response = self.secret_connector.dispatch(
            "Secret.get_data", {"secret_id": secret_id, "domain_id": domain_id}
        )
        return response["data"]

    def list_secrets_from_query(self, secret_filter, domain_id, **kwargs):
        secret_query = self._make_query(**secret_filter)
        response = self.list_secrets(secret_query, domain_id)

        if response.get("total_count", 0) == 0:
            resource_id = kwargs.get("resource_id")
            raise ERROR_RESOURCE_SECRETS_NOT_EXISTS(resource_id=resource_id)

        return response.get("results", [])

    def get_secret_data_from_plugin(
        self, plugin_info: dict, capability: dict, domain_id: str
    ) -> (dict, dict):
        plugin_id = plugin_info["plugin_id"]
        secret_id = plugin_info.get("secret_id")
        provider = plugin_info.get("provider")

        use_resource_secret = capability.get("use_resource_secret", False)
        supported_schema = capability.get("supported_schema", [])

        self._check_plugin_secret(use_resource_secret, plugin_info)

        if use_resource_secret:
            secret_query = self._make_query(
                supported_schema=supported_schema, provider=provider
            )
        else:
            secret_query = self._make_query(
                supported_schema=supported_schema, secret_id=secret_id
            )

        response = self.list_secrets(secret_query, domain_id)

        if response.get("total_count", 0) == 0:
            if use_resource_secret:
                raise ERROR_SUPPORTED_SECRETS_NOT_EXISTS(
                    plugin_id=plugin_id, provider=provider
                )
            else:
                raise ERROR_NOT_FOUND(key="plugin_info.secret_id", value=secret_id)

        result = response["results"][0]
        secret_id = result["secret_id"]
        schema_id = result.get("schema_id")

        return self.get_secret_data(secret_id, domain_id), schema_id

    @staticmethod
    def _check_plugin_secret(use_resource_secret: bool, plugin_info: dict) -> None:
        secret_id = plugin_info.get("secret_id")
        provider = plugin_info.get("provider")

        if use_resource_secret:
            if provider is None:
                raise ERROR_REQUIRED_PARAMETER(key="plugin_info.provider")
        else:
            if secret_id is None:
                raise ERROR_REQUIRED_PARAMETER(key="plugin_info.secret_id")

    @staticmethod
    def _make_query(**secret_filter):
        supported_schema = secret_filter.get("supported_schema")
        secret_id = secret_filter.get("secret_id")
        service_account_id = secret_filter.get("service_account_id")
        project_id = secret_filter.get("project_id")
        provider = secret_filter.get("provider")
        secrets = secret_filter.get("secrets")

        query = {"filter": []}

        if supported_schema:
            query["filter"].append({"k": "schema_id", "v": supported_schema, "o": "in"})

        if secret_id:
            query["filter"].append({"k": "secret_id", "v": secret_id, "o": "eq"})

        if provider:
            query["filter"].append({"k": "provider", "v": provider, "o": "eq"})

        if service_account_id:
            query["filter"].append(
                {"k": "service_account_id", "v": service_account_id, "o": "eq"}
            )

        if project_id:
            query["filter"].append({"k": "project_id", "v": project_id, "o": "eq"})

        if secrets:
            query["filter"].append({"k": "secret_id", "v": secrets, "o": "in"})

        return query



import copy
import logging

from spaceone.core import cache
from spaceone.core.manager import BaseManager

from spaceone.monitoring.conf.default_escalation_policy import DEFAULT_ESCALATION_POLICY
from spaceone.monitoring.error.escalation_policy import *
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy

_LOGGER = logging.getLogger(__name__)


class EscalationPolicyManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.escalation_policy_model: EscalationPolicy = self.locator.get_model(
            "EscalationPolicy"
        )

    def create_escalation_policy(self, params):
        def _rollback(vo: EscalationPolicy):
            _LOGGER.info(
                f"[create_escalation_policy._rollback] "
                f"Delete escalation policy : {vo.name} "
                f"({vo.escalation_policy_id})"
            )
            vo.delete()

        escalation_policy_vo: EscalationPolicy = self.escalation_policy_model.create(
            params
        )
        self.transaction.add_rollback(_rollback, escalation_policy_vo)

        return escalation_policy_vo

    @cache.cacheable(key="escalation-policy:{domain_id}:default:init", expire=300)
    def create_default_escalation_policy(
        self, domain_id, workspace_id, project_id=None
    ):
        default_escalation_policy = copy.deepcopy(DEFAULT_ESCALATION_POLICY)

        if project_id:
            default_escalation_policy["resource_group"] = "PROJECT"
            default_escalation_policy["domain_id"] = domain_id
            default_escalation_policy["workspace_id"] = workspace_id
            default_escalation_policy["project_id"] = project_id
        else:
            default_escalation_policy["resource_group"] = "WORKSPACE"
            default_escalation_policy["domain_id"] = domain_id
            default_escalation_policy["workspace_id"] = workspace_id
            default_escalation_policy["project_id"] = "*"

        return self.create_escalation_policy(default_escalation_policy)

    def update_escalation_policy_by_vo(
        self, params: dict, escalation_policy_vo: EscalationPolicy
    ) -> EscalationPolicy:
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_escalation_policy_by_vo._rollback] Revert Data : "
                f'{old_data["escalation_policy_id"]}'
            )
            escalation_policy_vo.update(old_data)

        self.transaction.add_rollback(_rollback, escalation_policy_vo.to_dict())

        updated_vo: EscalationPolicy = escalation_policy_vo.update(params)

        cache.delete(
            f"escalation-policy-condition:{updated_vo.domain_id}:{updated_vo.workspace_id}:{updated_vo.escalation_policy_id}"
        )

        return updated_vo

    def set_default_escalation_policy(self, params, escalation_policy_vo):
        global_escalation_policy_vos = self.escalation_policy_model.filter(
            domain_id=params["domain_id"],
            workspace_id=params["workspace_id"],
            resource_group="WORKSPACE",
        )

        for global_escalation_policy_vo in global_escalation_policy_vos:
            global_escalation_policy_vo.update({"is_default": False})

        return escalation_policy_vo.update({"is_default": True})

    def is_default_escalation_policy(self, domain_id, workspace_id, project_id=None):
        query = {
            "count_only": True,
            "filter": [
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "workspace_id", "v": workspace_id, "o": "eq"},
            ],
        }

        if project_id:
            query["filter"].append({"k": "project_id", "v": project_id, "o": "eq"})

        (
            escalation_policy_vos,
            total_count,
        ) = self.list_escalation_policies(query)

        if total_count == 0:
            return False
        return True

    def get_default_escalation_policy(
        self, workspace_id: str, domain_id: str, project_id: str
    ) -> EscalationPolicy:
        if not self.is_default_escalation_policy(workspace_id, domain_id, project_id):
            return self.create_default_escalation_policy(domain_id, workspace_id)
        else:
            # TODO: need to refactor
            query = {
                "filter": [
                    {"k": "domain_id", "v": domain_id, "o": "eq"},
                    {"k": "workspace_id", "v": workspace_id, "o": "eq"},
                    {"k": "project_id", "v": project_id, "o": "eq"},
                    {"k": "is_default", "v": True, "o": "eq"},
                ]
            }

            return self.escalation_policy_model.get(**query)

    def delete_escalation_policy(
        self,
        escalation_policy_id: str,
        workspace_id: str,
        domain_id: str,
        user_projects: list = None,
    ) -> None:
        conditions = {
            "escalation_policy_id": escalation_policy_id,
            "workspace_id": workspace_id,
            "domain_id": domain_id,
        }

        if user_projects:
            conditions["project_id"] = user_projects

        escalation_policy_vo: EscalationPolicy = self.get_escalation_policy(
            **conditions
        )

        if escalation_policy_vo.is_default:
            raise ERROR_DEFAULT_ESCALATION_POLICY_NOT_ALLOW_DELETION(
                escalation_policy_id=escalation_policy_id
            )

        cache.delete(
            f"escalation-policy-condition:{domain_id}:{workspace_id}:{escalation_policy_id}"
        )

        escalation_policy_vo.delete()

    def get_escalation_policy(
        self,
        escalation_policy_id: str,
        workspace_id: str,
        domain_id: str,
        user_projects: list = None,
    ) -> EscalationPolicy:
        conditions = {
            "escalation_policy_id": escalation_policy_id,
            "workspace_id": workspace_id,
            "domain_id": domain_id,
        }

        if user_projects:
            conditions["project_id"] = user_projects

        return self.escalation_policy_model.get(**conditions)

    def list_escalation_policies(self, query: dict) -> dict:
        return self.escalation_policy_model.query(**query)

    def stat_escalation_policies(self, query: dict) -> dict:
        return self.escalation_policy_model.stat(**query)



import logging
from fastapi import Request, Body
from fastapi_utils.inferring_router import InferringRouter
from fastapi_utils.cbv import cbv
from spaceone.core.fastapi.api import BaseAPI, exception_handler
from spaceone.core.error import *
from spaceone.core import config, cache
from spaceone.monitoring.model import Alert
from spaceone.monitoring.service import AlertService
from spaceone.monitoring.manager import IdentityManager
from fastapi.responses import RedirectResponse

_LOGGER = logging.getLogger(__name__)

router = InferringRouter()


@cbv(router)
class Alert(BaseAPI):
    service = "Alert"

    @router.get("/alert/{alert_id}/{access_key}/{state}")
    @exception_handler
    async def update_alert_state_get(self, alert_id: str, access_key: str, state: str):
        alert_info = self._update_alert_state(alert_id, access_key, state)
        domain_name = self._get_domain_name(alert_info["domain_id"])

        return self._make_redirect_response(alert_id, domain_name)

    @router.post("/alert/{alert_id}/{access_key}/{state}")
    async def update_alert_state_post(
        self, alert_id: str, access_key: str, state: str, request: Request
    ):
        params, metadata = await self.parse_request(request)

        if params.get("code") != "TIME_OUT":
            self._update_alert_state(alert_id, access_key, state)

            return {"detail": f"Alert({alert_id}) state was changed."}
        else:
            raise ERROR_REQUEST_TIMEOUT()

    def _update_alert_state(self, alert_id, access_key, state):
        alert_service: AlertService = self.locator.get_service("AlertService")

        alert_vo: Alert = alert_service.update_state(
            {"alert_id": alert_id, "access_key": access_key, "state": state}
        )

        return {
            "alert_number": alert_vo.alert_number,
            "alert_id": alert_vo.alert_id,
            "title": alert_vo.title,
            "state": alert_vo.state,
            "assignee": alert_vo.assignee,
            "urgency": alert_vo.urgency,
            "domain_id": alert_vo.domain_id,
        }

    @cache.cacheable(key="domain-name:{domain_id}", expire=3600)
    def _get_domain_name(self, domain_id):
        identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
        domain_info = identity_mgr.get_domain(domain_id)
        return domain_info["name"]

    @staticmethod
    def _make_redirect_response(alert_id, domain_name):
        console_domain = config.get_global("CONSOLE_DOMAIN")

        if console_domain.strip() != "":
            console_domain = console_domain.format(domain_name=domain_name)
            return RedirectResponse(f"{console_domain}/alert-manager/alert/{alert_id}")
        else:
            return None



import functools

from spaceone.api.monitoring.v1 import event_pb2
from spaceone.core import utils
from spaceone.core.pygrpc.message_type import *

from spaceone.monitoring.model.event_model import Event

__all__ = ["EventInfo", "EventsInfo"]


def EventResourceInfo(vo):
    if vo:
        info = {
            "resource_id": vo.resource_id,
            "resource_type": vo.resource_type,
            "name": vo.name,
        }

        return event_pb2.EventResource(**info)
    else:
        return None


def EventInfo(event_vo: Event, minimal=False):
    info = {
        "event_id": event_vo.event_id,
        "event_key": event_vo.event_key,
        "event_type": event_vo.event_type,
        "title": event_vo.title,
        "severity": event_vo.severity,
        "alert_id": event_vo.alert_id,
    }

    if not minimal:
        info.update(
            {
                "description": event_vo.description,
                "rule": event_vo.rule,
                "provider": event_vo.provider,
                "account": event_vo.account,
                "image_url": event_vo.image_url,
                "resource": EventResourceInfo(event_vo.resource),
                "raw_data": change_struct_type(event_vo.raw_data),
                "additional_info": change_struct_type(event_vo.additional_info),
                "alert_id": event_vo.alert_id,
                "webhook_id": event_vo.webhook_id,
                "project_id": event_vo.project_id,
                "workspace_id": event_vo.workspace_id,
                "domain_id": event_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(event_vo.created_at),
                "occurred_at": utils.datetime_to_iso8601(event_vo.occurred_at),
            }
        )

    return event_pb2.EventInfo(**info)


def EventsInfo(note_vos, total_count, **kwargs):
    return event_pb2.EventsInfo(
        results=list(map(functools.partial(EventInfo, **kwargs), note_vos)),
        total_count=total_count,
    )



import functools

from spaceone.api.monitoring.v1 import webhook_pb2
from spaceone.core import config
from spaceone.core import utils
from spaceone.core.pygrpc.message_type import *

from spaceone.monitoring.model.webhook_model import Webhook

__all__ = ["WebhookInfo", "WebhooksInfo"]


def PluginInfo(vo):
    if vo:
        info = {
            "plugin_id": vo.plugin_id,
            "version": vo.version,
            "options": change_struct_type(vo.options),
            "metadata": change_struct_type(vo.metadata),
            "upgrade_mode": vo.upgrade_mode,
        }

        return webhook_pb2.WebhookPluginInfo(**info)
    else:
        return None


def WebhookInfo(webhook_vo: Webhook, minimal=False):
    if webhook_vo.webhook_url:
        webhook_url = f'{config.get_global("WEBHOOK_DOMAIN")}{webhook_vo.webhook_url}'
    else:
        webhook_url = None

    info = {
        "webhook_id": webhook_vo.webhook_id,
        "name": webhook_vo.name,
        "state": webhook_vo.state,
        "webhook_url": webhook_url,
        "project_id": webhook_vo.project_id,
        "workspace_id": webhook_vo.workspace_id,
    }

    if not minimal:
        info.update(
            {
                "access_key": webhook_vo.access_key,
                "capability": change_struct_type(webhook_vo.capability),
                "plugin_info": PluginInfo(webhook_vo.plugin_info),
                "domain_id": webhook_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(webhook_vo.created_at),
            }
        )

    return webhook_pb2.WebhookInfo(**info)


def WebhooksInfo(webhook_vos, total_count, **kwargs):
    return webhook_pb2.WebhooksInfo(
        results=list(map(functools.partial(WebhookInfo, **kwargs), webhook_vos)),
        total_count=total_count,
    )



import functools
from typing import List

from spaceone.api.monitoring.v1 import event_rule_pb2
from spaceone.core import utils
from spaceone.core.pygrpc.message_type import *

from spaceone.monitoring.model.event_rule_model import (
    EventRule,
    EventRuleCondition,
    EventRuleOptions,
)

__all__ = ["EventRuleInfo", "EventRulesInfo"]


def EventRuleConditionsInfo(condition_vos: List[EventRuleCondition]):
    if condition_vos is None:
        condition_vos = []

    conditions_info = []

    for vo in condition_vos:
        info = {"key": vo.key, "value": vo.value, "operator": vo.operator}

        conditions_info.append(event_rule_pb2.EventRuleCondition(**info))

    return conditions_info


def EventRuleActionsInfo(actions_data):
    if actions_data is None:
        return None
    else:
        info = {}

        for key, value in actions_data.items():
            if key == "add_additional_info":
                info[key] = change_struct_type(value)
            else:
                info[key] = value

        return event_rule_pb2.EventRuleActions(**info)


def EventRuleOptionsInfo(vo: EventRuleOptions):
    if vo is None:
        return None
    else:
        info = {"stop_processing": vo.stop_processing}

        return event_rule_pb2.EventRuleOptions(**info)


def EventRuleInfo(event_rule_vo: EventRule, minimal=False):
    info = {
        "event_rule_id": event_rule_vo.event_rule_id,
        "name": event_rule_vo.name,
        "order": event_rule_vo.order,
        "resource_group": event_rule_vo.resource_group,
        "project_id": event_rule_vo.project_id,
    }

    if not minimal:
        info.update(
            {
                "conditions": EventRuleConditionsInfo(event_rule_vo.conditions),
                "conditions_policy": event_rule_vo.conditions_policy,
                "actions": EventRuleActionsInfo(event_rule_vo.actions),
                "options": EventRuleOptionsInfo(event_rule_vo.options),
                "tags": change_struct_type(event_rule_vo.tags),
                "domain_id": event_rule_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(event_rule_vo.created_at),
            }
        )

    return event_rule_pb2.EventRuleInfo(**info)


def EventRulesInfo(event_rule_vos, total_count, **kwargs):
    return event_rule_pb2.EventRulesInfo(
        results=list(map(functools.partial(EventRuleInfo, **kwargs), event_rule_vos)),
        total_count=total_count,
    )



from spaceone.api.monitoring.v1 import escalation_policy_pb2, escalation_policy_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class EscalationPolicy(BaseAPI, escalation_policy_pb2_grpc.EscalationPolicyServicer):
    pb2 = escalation_policy_pb2
    pb2_grpc = escalation_policy_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            return self.locator.get_info(
                "EscalationPolicyInfo", escalation_policy_service.create(params)
            )

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            return self.locator.get_info(
                "EscalationPolicyInfo", escalation_policy_service.update(params)
            )

    def set_default(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            return self.locator.get_info(
                "EscalationPolicyInfo", escalation_policy_service.set_default(params)
            )

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            escalation_policy_service.delete(params)
            return self.locator.get_info("EmptyInfo")

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            return self.locator.get_info(
                "EscalationPolicyInfo", escalation_policy_service.get(params)
            )

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            escalation_policy_vos, total_count = escalation_policy_service.list(params)
            return self.locator.get_info(
                "EscalationPoliciesInfo",
                escalation_policy_vos,
                total_count,
                minimal=self.get_minimal(params),
            )

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service(
            "EscalationPolicyService", metadata
        ) as escalation_policy_service:
            return self.locator.get_info(
                "StatisticsInfo", escalation_policy_service.stat(params)
            )



import logging

from spaceone.core.connector.space_connector import SpaceConnector
from spaceone.core.manager import BaseManager

from spaceone.monitoring.error import *

_LOGGER = logging.getLogger(__name__)


class RepositoryManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.repo_connector: SpaceConnector = self.locator.get_connector(
            "SpaceConnector", service="repository"
        )

    def get_plugin(self, plugin_id: str, domain_id: str):
        return self.repo_connector.dispatch(
            "Plugin.get",
            {"plugin_id": plugin_id, "domain_id": domain_id},
        )

    def check_plugin_version(self, plugin_id, version, domain_id):
        response = self.repo_connector.dispatch(
            "Plugin.get_versions", {"plugin_id": plugin_id, "domain_id": domain_id}
        )
        versions = response.get("results", [])

        if version not in versions:
            raise ERROR_INVALID_PLUGIN_VERSION(plugin_id=plugin_id, version=version)



from spaceone.monitoring.model.alert_model import Alert, AlertNumber
from spaceone.monitoring.model.data_source_model import DataSource
from spaceone.monitoring.model.escalation_policy_model import EscalationPolicy
from spaceone.monitoring.model.event_model import Event
from spaceone.monitoring.model.event_rule_model import EventRule
from spaceone.monitoring.model.job_model import Job
from spaceone.monitoring.model.note_model import Note
from spaceone.monitoring.model.plugin_metadata_model import *
from spaceone.monitoring.model.project_alert_config_model import ProjectAlertConfig
from spaceone.monitoring.model.webhook_model import Webhook



from spaceone.api.monitoring.v1 import metric_pb2
from spaceone.core.pygrpc.message_type import *

__all__ = ["MetricsInfo", "MetricDataInfo"]


def MetricDataInfo(metric_data):
    info = {
        "labels": change_list_value_type(metric_data.get("labels", [])),
        "values": change_struct_type(metric_data["values"]),
        "domain_id": metric_data["domain_id"],
    }

    return metric_pb2.MetricDataInfo(**info)


def MetricInfo(metric):
    info = {
        "key": metric["key"],
        "name": metric["name"],
    }

    if "group" in metric:
        info.update({"group": metric["group"]})

    if "unit" in metric:
        info.update({"unit": change_struct_type(metric["unit"])})

    if "metric_query" in metric:
        info.update({"metric_query": change_struct_type(metric["metric_query"])})

    return metric_pb2.MetricInfo(**info)


def MetricsInfo(metrics_info):
    info = {
        "metrics": list(map(MetricInfo, metrics_info["metrics"]))
        if metrics_info["metrics"]
        else None,
        "available_resources": change_struct_type(metrics_info["available_resources"]),
        "domain_id": metrics_info["domain_id"],
    }

    return metric_pb2.MetricsInfo(**info)



import functools

from spaceone.api.monitoring.v1 import project_alert_config_pb2
from spaceone.core import utils

from spaceone.monitoring.info.escalation_policy_info import EscalationPolicyInfo
from spaceone.monitoring.model.project_alert_config_model import (
    ProjectAlertConfig,
    AlertOptions,
)

__all__ = ["ProjectAlertConfigInfo", "ProjectAlertConfigsInfo"]


def AlertOptionsInfo(vo: AlertOptions):
    if vo:
        info = {
            "notification_urgency": vo.notification_urgency,
            "recovery_mode": vo.recovery_mode,
        }

        return project_alert_config_pb2.AlertOptions(**info)
    else:
        return None


def ProjectAlertConfigInfo(project_alert_config_vo: ProjectAlertConfig, minimal=False):
    info = {
        "project_id": project_alert_config_vo.project_id,
        "workspace_id": project_alert_config_vo.workspace_id,
        "options": AlertOptionsInfo(project_alert_config_vo.options),
    }

    if not minimal:
        info.update(
            {
                "domain_id": project_alert_config_vo.domain_id,
                "created_at": utils.datetime_to_iso8601(
                    project_alert_config_vo.created_at
                ),
            }
        )

        if project_alert_config_vo.escalation_policy:
            info["escalation_policy_info"] = EscalationPolicyInfo(
                project_alert_config_vo.escalation_policy, minimal=True
            )

    return project_alert_config_pb2.ProjectAlertConfigInfo(**info)


def ProjectAlertConfigsInfo(project_alert_config_vos, total_count, **kwargs):
    return project_alert_config_pb2.ProjectAlertConfigsInfo(
        results=list(
            map(
                functools.partial(ProjectAlertConfigInfo, **kwargs),
                project_alert_config_vos,
            )
        ),
        total_count=total_count,
    )



from pydantic import BaseModel

__all__ = ["WebhookInitRequest", "WebhookVerityRequest"]


class WebhookInitRequest(BaseModel):
    options: dict


class WebhookVerityRequest(BaseModel):
    options: dict



import functools
import logging
from typing import List

from spaceone.core import utils
from spaceone.core.manager import BaseManager

# from spaceone.monitoring.manager import IdentityManager
from spaceone.monitoring.model.event_rule_model import EventRule, EventRuleCondition

_LOGGER = logging.getLogger(__name__)


class EventRuleManager(BaseManager):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.event_rule_model: EventRule = self.locator.get_model("EventRule")
        # self.identity_mgr: IdentityManager = self.locator.get_manager("IdentityManager")
        self._service_account_info = {}

    def create_event_rule(self, params: dict) -> EventRule:
        def _rollback(vo: EventRule):
            _LOGGER.info(
                f"[create_event_rule._rollback] "
                f"Delete event rule : {vo.name} "
                f"({vo.event_rule_id})"
            )
            vo.delete()

        event_rule_vo: EventRule = self.event_rule_model.create(params)
        self.transaction.add_rollback(_rollback, event_rule_vo)

        return event_rule_vo

    def update_event_rule(self, params):
        event_rule_vo: EventRule = self.get_event_rule(
            params["event_rule_id"], params["domain_id"], params["workspace_id"]
        )
        return self.update_event_rule_by_vo(params, event_rule_vo)

    def update_event_rule_by_vo(self, params: dict, event_rule_vo: EventRule):
        def _rollback(old_data: dict):
            _LOGGER.info(
                f"[update_event_rule_by_vo._rollback] Revert Data : "
                f'{old_data["event_rule_id"]}'
            )
            event_rule_vo.update(old_data)

        self.transaction.add_rollback(_rollback, event_rule_vo.to_dict())

        return event_rule_vo.update(params)

    def delete_event_rule(self, event_rule_id, domain_id):
        event_rule_vo: EventRule = self.get_event_rule(event_rule_id, domain_id)
        self.delete_event_rule_by_vo(event_rule_vo)

    def delete_event_rule_by_vo(self, event_rule_vo):
        event_rule_vo.delete()

    def get_event_rule(
        self,
        event_rule_id: str,
        domain_id: str,
        workspace_id: str,
        user_projects: list = None,
    ):
        conditions = {
            "event_rule_id": event_rule_id,
            "domain_id": domain_id,
            "workspace_id": workspace_id,
        }

        if user_projects:
            conditions["project_id"] = user_projects

        return self.event_rule_model.get(**conditions)

    def list_event_rules(self, query: dict) -> dict:
        return self.event_rule_model.query(**query)

    def stat_event_rules(self, query: dict) -> dict:
        return self.event_rule_model.stat(**query)

    def change_event_data(self, event_data, project_id, domain_id, workspace_id):
        event_rule_vos: List[EventRule] = self._get_project_event_rules(
            project_id, domain_id, workspace_id
        )

        for event_rule_vo in event_rule_vos:
            is_match = self._change_event_data_by_event_rule(event_data, event_rule_vo)

            if is_match:
                event_data = self._change_event_data_with_actions(
                    event_data, event_rule_vo.actions, domain_id
                )

            if is_match and event_rule_vo.options.stop_processing:
                break

        # TODO: Check Global Event Rule

        return event_data

    def _change_event_data_with_actions(self, event_data, actions, domain_id):
        for action, value in actions.items():
            if action == "change_project":
                event_data["project_id"] = value

            if action == "change_assignee":
                event_data["assignee"] = value

            if action == "change_urgency":
                event_data["urgency"] = value

            if action == "add_additional_info":
                event_data["additional_info"] = event_data.get("additional_info", {})
                event_data["additional_info"].update(value)

            if action == "no_notification":
                event_data["no_notification"] = value

            # TODO: Refactor Match Service Account
            # if action == "match_service_account":
            #     source = value["source"]
            #     target_key = value["target"]
            #     target_value = utils.get_dict_value(event_data, source)
            #     if target_value:
            #         service_account_info = self._get_service_account(
            #             target_key, target_value, domain_id
            #         )
            #         if service_account_info:
            #             event_data["project_id"] = service_account_info.get(
            #                 "project_info", {}
            #             ).get("project_id")

        return event_data

    def _change_event_data_by_event_rule(self, event_data, event_rule_vo: EventRule):
        conditions_policy = event_rule_vo.conditions_policy

        if conditions_policy == "ALWAYS":
            return True
        else:
            results = list(
                map(
                    functools.partial(self._check_condition, event_data),
                    event_rule_vo.conditions,
                )
            )

            if conditions_policy == "ALL":
                return all(results)
            else:
                return any(results)

    @staticmethod
    def _check_condition(event_data, condition: EventRuleCondition):
        event_value = utils.get_dict_value(event_data, condition.key)
        condition_value = condition.value
        operator = condition.operator

        if event_value is None:
            return False

        if operator == "eq":
            if event_value == condition_value:
                return True
            else:
                return False
        elif operator == "contain":
            if event_value.lower().find(condition_value.lower()) >= 0:
                return True
            else:
                return False
        elif operator == "not":
            if event_value != condition_value:
                return True
            else:
                return False
        elif operator == "not_contain":
            if event_value.lower().find(condition_value.lower()) < 0:
                return True
            else:
                return False

        return False

    def _get_project_event_rules(self, project_id, domain_id, workspace_id):
        query = {
            "filter": [
                {"k": "project_id", "v": project_id, "o": "eq"},
                {"k": "domain_id", "v": domain_id, "o": "eq"},
                {"k": "workspace_id", "v": workspace_id, "o": "eq"},
            ],
            "sort": {"key": "order"},
        }

        event_rule_vos, total_count = self.list_event_rules(query)
        return event_rule_vos

    # def _get_service_account(self, target_key, target_value, domain_id):
    #     if f"{domain_id}:{target_key}:{target_value}" in self._service_account_info:
    #         return self._service_account_info[
    #             f"{domain_id}:{target_key}:{target_value}"
    #         ]
    #
    #     query = {
    #         "filter": [{"k": target_key, "v": target_value, "o": "eq"}],
    #         "only": ["service_account_id", "project_info"],
    #     }
    #
    #     response = self.identity_mgr.list_service_accounts(query, domain_id)
    #     results = response.get("results", [])
    #     total_count = response.get("total_count", 0)
    #
    #     service_account_info = None
    #     if total_count > 0:
    #         service_account_info = results[0]
    #
    #     self._service_account_info[
    #         f"{domain_id}:{target_key}:{target_value}"
    #     ] = service_account_info
    #     return service_account_info



from spaceone.monitoring.manager.alert_manager import AlertManager
from spaceone.monitoring.manager.data_source_manager import DataSourceManager
from spaceone.monitoring.manager.data_source_plugin_manager import (
    DataSourcePluginManager,
)
from spaceone.monitoring.manager.escalation_policy_manager import (
    EscalationPolicyManager,
)
from spaceone.monitoring.manager.event_manager import EventManager
from spaceone.monitoring.manager.event_rule_manager import EventRuleManager
from spaceone.monitoring.manager.identity_manager import IdentityManager
from spaceone.monitoring.manager.inventory_manager import InventoryManager
from spaceone.monitoring.manager.job_manager import JobManager
from spaceone.monitoring.manager.note_manager import NoteManager
from spaceone.monitoring.manager.notification_manager import NotificationManager
from spaceone.monitoring.manager.plugin_manager import PluginManager
from spaceone.monitoring.manager.project_alert_config_manager import (
    ProjectAlertConfigManager,
)
from spaceone.monitoring.manager.repository_manager import RepositoryManager
from spaceone.monitoring.manager.secret_manager import SecretManager
from spaceone.monitoring.manager.webhook_manager import WebhookManager
from spaceone.monitoring.manager.webhook_plugin_manager import WebhookPluginManager
